unit FmaApi;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Main header file
//
// FmaApi.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 27/Aug/1999
// Updated: 18/Oct/2002
//
// History: 17/Jan/1999 kvitekp - Initially created for IMA project.
//
//          29/Jan/2000 kvitekp - Added FmaDupeMsg(pfmm)
//
//          09/Feb/2000 akzhan  - Initially converted to Delphi
//                                FMP_XXX constants are declared as
//                                variables these initialized at startup.
//                                'type' member of FMM struct was renamed
//                                to '_type' member of FMM record.
//                                FmaAddFmpFormat and fmaInit isn't supported.
//          10/Feb/2000 kvitekp - Added FmaAddFmpFormatV()
//          10/Feb/2000 akzhan  - Added FmaAddFmpFormatV and FmaAddFmpFormat
//          11/Feb/2000 kvitekp - Added Pkt base stuff
//          14/Feb/2000 akzhan  - Added Pkt base stuff
//          18/Feb/2000 akzhan  - FmaReadMsg and FmaWriteMsg definitions changed,
//                                FmaAddFmpFormat is excluded
//          11/Feb/2001 akzhan  - Fidonet mail message property ids
//                                redeclared as constants.
//                                Unit declared as weak packaged.
//          14/Feb/2001 akzhan  - FmaCreateMsgMap redefined.
//          18/Oct/2002 Plyako A- FMAERROR_CALLBACK_CANCELLED error code added
//
/////////////////////////////////////////////////////////////////////////////

{$I FmxDef.inc}
{$IFNDEF VIRTUALPASCAL}
{$WEAKPACKAGEUNIT ON}
{$ENDIF}
interface

uses
  Windows, FmaDef;

/////////////////////////////////////////////////////////////////////////////
// Fidonet mail message property declarations
/////////////////////////////////////////////////////////////////////////////

  // Fidonet mail message property structure

type
  PFMP = ^FMP;
  FMP = packed record         (* fmp *)
    pfmpPrev: PFMP;                   // Pointer to prev property
    pfmpNext: PFMP;                   // Pointer to next property
    id: ULONG;                         // Property id (FMP_)
    cb: ULONG;                         // Property data size
  //ab: array [cb] of BYTE;            // Property data
  end;

  // Macros to manage fidonet mail message property data

function PFMPDATA(pfmp: PFMP): PBYTE; // Pointer to property data
function PFMPLONG(pfmp: PFMP): PLONG; // Pointer to property long
function PFMPPSZ(pfmp: PFMP): PAnsiChar;  // Pointer to property psz
function PFMPDTTM(pfmp: PFMP): PDTTM; // Pointer to property dttm

  // Fidonet mail message property id masks
const
  FMP_FLGMASK =    $ff000000;     // Flags mask
  FMP_POSMASK =    $00ff0000;     // Position mask
  FMP_TAGMASK =    $0000ffff;     // Tag mask

  // Fidonet mail message property id macros

function FMP_FLG(id: DWORD): DWORD;
function FMP_POS(id: DWORD): DWORD;
function FMP_TAG(id: DWORD): DWORD;

  // Fidonet mail message property id flags
const
  FMP_ASZ     =     $01000000;    // Asciiz string
  FMP_PSZ     =     $02000000;    // Pointer(s) to asciiz string(s)
  FMP_U32     =     $04000000;    // Unsigned 32-bit value(s)
  FMP_BIN     =     $08000000;    // Counted binary of cb size
  FMP_DTM     =     $10000000;    // Date/time info (DTTM format)
  FMP_EOM     =     $20000000;    // Goes past the message text
  FMP_ONE     =     $40000000;    // Only one instance is allowed
  FMP_KLU = (FMP_ASZ or $80000000); // FNT kludge property

  // Macro to define property ids

function _FMP(tag, pos, flg: DWORD): DWORD;

const
  // Fidonet mail message basic property ids
  FMP_MESSAGETEXT   = (0 and FMP_TAGMASK) or ((0 shl 16) and FMP_POSMASK) or ((FMP_PSZ  or  FMP_ONE) and FMP_FLGMASK); // _FMP( 0, 0, FMP_PSZ  or  FMP_ONE);        // Message text
  FMP_SENDERNAME    = (1 and FMP_TAGMASK) or ((1 shl 16) and FMP_POSMASK) or ((FMP_ASZ  or  FMP_ONE) and FMP_FLGMASK); // _FMP( 1, 1, FMP_ASZ  or  FMP_ONE);        // Author's name
  FMP_RECEVRNAME    = (2 and FMP_TAGMASK) or ((2 shl 16) and FMP_POSMASK) or ((FMP_ASZ  or  FMP_ONE) and FMP_FLGMASK); // _FMP( 2, 2, FMP_ASZ  or  FMP_ONE);        // Recipient's name
  FMP_ORIGADDR      = (3 and FMP_TAGMASK) or ((3 shl 16) and FMP_POSMASK) or ((FMP_ASZ  or  FMP_ONE) and FMP_FLGMASK); // _FMP( 3, 3, FMP_ASZ  or  FMP_ONE);        // Originating address
  FMP_DESTADDR      = (4 and FMP_TAGMASK) or ((4 shl 16) and FMP_POSMASK) or ((FMP_ASZ  or  FMP_ONE) and FMP_FLGMASK); // _FMP( 4, 4, FMP_ASZ  or  FMP_ONE);        // Destination address
  FMP_GATEADDR      = (5 and FMP_TAGMASK) or ((5 shl 16) and FMP_POSMASK) or ((FMP_ASZ  or  FMP_ONE) and FMP_FLGMASK); // _FMP( 5, 5, FMP_ASZ  or  FMP_ONE);        // Gate address
  FMP_SUBJECT       = (6 and FMP_TAGMASK) or ((6 shl 16) and FMP_POSMASK) or ((FMP_ASZ  or  FMP_ONE) and FMP_FLGMASK); // _FMP( 6, 6, FMP_ASZ  or  FMP_ONE);        // Message subject
  FMP_FILEATTACH    = (7 and FMP_TAGMASK) or ((7 shl 16) and FMP_POSMASK) or ((FMP_ASZ             ) and FMP_FLGMASK); // _FMP( 7, 7, FMP_ASZ          );           // File attachment
  FMP_FILEREQUEST   = (8 and FMP_TAGMASK) or ((8 shl 16) and FMP_POSMASK) or ((FMP_ASZ             ) and FMP_FLGMASK); // _FMP( 8, 8, FMP_ASZ          );           // File request
  FMP_FILEUPDATEREQ = (9 and FMP_TAGMASK) or ((9 shl 16) and FMP_POSMASK) or ((FMP_ASZ             ) and FMP_FLGMASK); // _FMP( 9, 9, FMP_ASZ          );           // File update request
  // Fidonet mail message additional property ids
  FMP_DTTMWRITTEN   = (11 and FMP_TAGMASK) or ((11 shl 16) and FMP_POSMASK) or ((FMP_DTM  or  FMP_ONE) and FMP_FLGMASK); // _FMP(11,11, FMP_DTM  or  FMP_ONE);        // When msg was written
  FMP_DTTMRECEIVED  = (12 and FMP_TAGMASK) or ((12 shl 16) and FMP_POSMASK) or ((FMP_DTM  or  FMP_ONE) and FMP_FLGMASK); // _FMP(12,12, FMP_DTM  or  FMP_ONE);        // When msg was received
  FMP_DTTMPROCESSED = (13 and FMP_TAGMASK) or ((13 shl 16) and FMP_POSMASK) or ((FMP_DTM  or  FMP_ONE) and FMP_FLGMASK); // _FMP(13,13, FMP_DTM  or  FMP_ONE);        // When msg was processed
  FMP_MSGREPLYTO    = (14 and FMP_TAGMASK) or ((14 shl 16) and FMP_POSMASK) or ((FMP_U32  or  FMP_ONE) and FMP_FLGMASK); // _FMP(14,14, FMP_U32  or  FMP_ONE);        // Msg which this msg replies to
  FMP_MSGREPLIES    = (15 and FMP_TAGMASK) or ((15 shl 16) and FMP_POSMASK) or ((FMP_U32  or  FMP_ONE) and FMP_FLGMASK); // _FMP(15,15, FMP_U32  or  FMP_ONE);        // Msgs which reply to this msg
  FMP_MSGREPLYNEXT  = (16 and FMP_TAGMASK) or ((16 shl 16) and FMP_POSMASK) or ((FMP_U32  or  FMP_ONE) and FMP_FLGMASK); // _FMP(16,16, FMP_U32  or  FMP_ONE);        // Next msg in reply chain if any
  FMP_TIMESREAD     = (17 and FMP_TAGMASK) or ((17 shl 16) and FMP_POSMASK) or ((FMP_U32  or  FMP_ONE) and FMP_FLGMASK); // _FMP(17,17, FMP_U32  or  FMP_ONE);        // Number of times message was read
  FMP_PASSWORDCRC   = (18 and FMP_TAGMASK) or ((18 shl 16) and FMP_POSMASK) or ((FMP_U32  or  FMP_ONE) and FMP_FLGMASK); // _FMP(18,18, FMP_U32  or  FMP_ONE);        // CRC-32 of password to access msg
  FMP_MESSAGECOST   = (19 and FMP_TAGMASK) or ((18 shl 16) and FMP_POSMASK) or ((FMP_U32  or  FMP_ONE) and FMP_FLGMASK); // _FMP(19,19, FMP_U32  or  FMP_ONE);        // Cost of message
  // Fidonet mail message kludge property ids.
  // Note: ^aFMPT,^aTOPT,^aINTL and ^aDOMAIN kludge info should be processed and
  // placed in appropriate properties. Also, all ^aFLAGS flags which have matching
  // FMMA_ attributes should not be stored in FMP_FLAGS to prevent ambiguities
  FMP_AREA          = (29 and FMP_TAGMASK) or ((29 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(29,29, FMP_KLU  or  FMP_ONE);        // FTN ^aAREA kludge info
  FMP_MSGID         = (30 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(30,30, FMP_KLU  or  FMP_ONE);        // FTN ^aMSGID kludge info
  FMP_REPLYID       = (31 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(31,30, FMP_KLU  or  FMP_ONE);        // FTN ^aREPLYID kludge info
  FMP_PID           = (32 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(32,30, FMP_KLU  or  FMP_ONE);        // FTN ^aPID kludge info
  FMP_TID           = (33 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(33,30, FMP_KLU  or  FMP_ONE);        // FTN ^aTID kludge info
  FMP_TZUTC         = (34 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(34,30, FMP_KLU  or  FMP_ONE);        // FTN ^aTZUTC kludge info ([+ or -]NNNN);
  FMP_FLAGS         = (35 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(35,30, FMP_KLU  or  FMP_ONE);        // FTN ^aFLAGS kludge info
  FMP_CHARSET       = (36 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(36,30, FMP_KLU  or  FMP_ONE);        // FTN ^aCHRS kludge info
  FMP_REALNAME      = (37 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(37,30, FMP_KLU  or  FMP_ONE);        // FTN ^aREALNAME kludge info
  FMP_REPLYADDR     = (38 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(38,30, FMP_KLU  or  FMP_ONE);        // FTN ^aREPLYADDR kludge info
  FMP_REPLYTO       = (39 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_ONE) and FMP_FLGMASK); // _FMP(39,30, FMP_KLU  or  FMP_ONE);        // FTN ^aREPLYTO kludge info
  FMP_FTSKLUDGE     = (40 and FMP_TAGMASK) or ((30 shl 16) and FMP_POSMASK) or ((FMP_KLU             ) and FMP_FLGMASK); // _FMP(40,30, FMP_KLU          );        // Unknown kludge line

  FMP_SEENBY2D      = (70 and FMP_TAGMASK) or ((70 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_EOM) and FMP_FLGMASK); // _FMP(70,70, FMP_KLU  or  FMP_EOM);        // 2D (net/node); SEEN-BY information
  FMP_PATH2D        = (71 and FMP_TAGMASK) or ((71 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_EOM) and FMP_FLGMASK); // _FMP(71,71, FMP_KLU  or  FMP_EOM);        // 2D (net/node); PATH information
  FMP_EOMKLUDGE     = (80 and FMP_TAGMASK) or ((80 shl 16) and FMP_POSMASK) or ((FMP_KLU  or  FMP_EOM) and FMP_FLGMASK); // _FMP(80,80, FMP_KLU  or  FMP_EOM);        // Unknown kludge line at eom

/////////////////////////////////////////////////////////////////////////////
// Fidonet mail message declarations
/////////////////////////////////////////////////////////////////////////////

  // Fidonet mail message object structure

type
  FMM = packed record (* fmm *)
    cb: UINT    ;                         // Size of this structure
    _type: UINT   ;                       // Msg native type (FMMT_)
    umsg: ULONG   ;                       // Unique message id or number
    attr: ULONG   ;                       // Msg attributes (MA_)
    pfmp: PFMP    ;                       // Msg property list
{$IFDEF FMM_EXTENSION}
    FMM_EXTENSION                         // Extension declarations
{$ENDIF}
  end;
  PFMM = ^FMM;
  TMsgObject = FMM;
  PMsgObject = PFMM;

  // Fidonet mail message types

const
  FMMT_NATIVE  =    0;              // Msg is native fmm message
  FMMT_FTS     =    1;              // Msg is from FTS-001 message base
  FMMT_SQUISH  =    2;              // Msg is from Squish message base
  FMMT_JAM     =    3;              // Msg is from JAM message base
  FMMT_PKT     =    4;              // Msg is from Pkt message base

  // Fidonet mail message attributes in fmm.attr

const
  MA_LOCAL        = $00000001;    // Msg created locally
  MA_INTRANSIT    = $00000002;    // Msg is in-transit
  MA_PRIVATE      = $00000004;    // Private message
  MA_READ         = $00000008;    // Read by addressee
  MA_SENT         = $00000010;    // Sent to remote
  MA_KILLSENT     = $00000020;    // Kill when sent
  MA_ARCHIVESENT  = $00000040;    // Archive when sent
  MA_HOLD         = $00000080;    // Hold for pick-up
  MA_CRASH        = $00000100;    // Crash
  MA_IMMEDIATE    = $00000200;    // Send Msg now, ignore restrictions
  MA_DIRECT       = $00000400;    // Send directly to destination
  MA_GATE         = $00000800;    // Send via gateway
  MA_FILEREQUEST  = $00001000;    // File request
  MA_FILEATTACH   = $00002000;    // File(s) attached to Msg
  MA_TRUNCFILE    = $00004000;    // Truncate file(s) when sent
  MA_KILLFILE     = $00008000;    // Delete file(s) when sent
  MA_RECEIPTREQ   = $00010000;    // Return receipt requested
  MA_CONFIRMREQ   = $00020000;    // Confirmation receipt requested
  MA_ORPHAN       = $00040000;    // Unknown destination
  MA_ENCRYPT      = $00080000;    // Msg text is encrypted
  MA_COMPRESS     = $00100000;    // Msg text is compressed
  MA_ESCAPED      = $00200000;    // Msg text is seven bit ASCII
  MA_FORCEPICKUP  = $00400000;    // Force pickup
  MA_TYPELOCAL    = $00800000;    // Msg is for local use only (not for export)
  MA_TYPEECHO     = $01000000;    // Msg is for conference distribution
  MA_TYPENET      = $02000000;    // Msg is direct network mail
  MA_SCANNED      = $04000000;    // Msg is scanned (squish)
  MA_FILEUPDATEREQ= $08000000;    // File update request
  MA_AUDITREQUEST = $10000000;    // Audit request
  MA_NODISPLAY    = $20000000;    // Msg may not be displayed to user
  MA_LOCKED       = $40000000;    // Msg is locked, no editing possible
  MA_DELETED      = $80000000;    // Msg is deleted

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaCreateMsg(cbExtra: UINT): PFMM; stdcall;
function FmaDestroyMsg(pfmm: PFMM): BOOL; stdcall;
function FmaDupeMsg(pfmm: PFMM): PFMM; stdcall;
function FmaDumpMsg(pfmm: PFMM): BOOL; stdcall;

function FmaGetFmp(pfmm: PFMM; id: ULONG; ifmp: UINT): PFMP; stdcall;
function FmaDelFmp(pfmm: PFMM; id: ULONG; ifmp: UINT): BOOL; stdcall;
function FmaDumpFmp(pfmp: PFMP): BOOL; stdcall;
function FmaAddFmp(pfmm: PFMM; id: ULONG; pbData: Pointer; cbData: ULONG): PFMP; stdcall;
function FmaAddFmpFormatV(pfmm: PFMM; id: ULONG; pszFormat: PAnsiChar; parg: Tva_list): PFMP; stdcall;
function FmaGetFmpTextLen(pfmp: PFMP): ULONG; stdcall;
{$ENDIF}
/////////////////////////////////////////////////////////////////////////////
// Abstract message base api declarations
/////////////////////////////////////////////////////////////////////////////

  // FmaOpenArea() open modes

const
  OPENAREA_EXISTING      =     0;   // Open existing, fail if does not exist
  OPENAREA_ALWAYS        =     1;   // Open existing or create if does not exist
  OPENAREA_CREATE_NEW    =     2;   // Create new, fail if already exists
  OPENAREA_CREATE_ALWAYS =     3;   // Create new, truncate if exists

  // FmaOpenArea() flags

const
  FOA_LOCKAREA     = $00000001;   // Lock area upon opening
  FOA_READONLY     = $00000002;   // Open area in readonly mode
  FOA_DENYALL      = $00000004;   // Open area in exclusive mode
  FOA_NETMAIL      = $00000100;   // Area contains netmail
  FOA_ECHOMAIL     = $00000200;   // Area contains echomail
  FOA_HIGHREAD     = $00000800;   // Maintain highest read message (Fts/Squish)
  FOA_KEEPUMSG     = $00010000;   // Keep unique message id in smh.umsg (Squish)
  FOA_DELAYTRIMMING= $00020000;   // Delay maxmsg trimming until close area (Squish)
  FOA_CREATEATEOF  = $00040000;   // Create new msgs at end of base (Squish)

  // FmaEnumMsgs() callback info structure

type
  ENUMMSGINFO = packed record (* emi *)
    msgn: TMsgNumber;                        // Message number
    umsg: UMSG;                        // Message uid
    imsg: ULONG;                       // Message index
    offs: ULONG;                       // Message offset (sqi.lfm or jdx.lmh)
    hash: ULONG;                       // Message hash (sqi.hash or jdx.crc)
    harea: TAreaHandle;                      // Message area handle
  end;
  PENUMMSGINFO = ^ENUMMSGINFO;

  // FmaEnumMsgs() callback function prototype

type
  PFNENUMMSGS = function(var pemi: ENUMMSGINFO; pvCallbackData: Pointer): BOOL stdcall;

  // FmaEnumMsgs() flags

const
  FENUMMSGS_REVERSE  = $00000001;  // Enumerate in reverse order
  FENUMMSGS_SAFEENUM = $00010000;  // Allows message map modification

  // FmaOpenMsg() open modes

const
  OPENMSG_CREATE     = 0;           // Create new msg for reading and writing, truncate if exists
  OPENMSG_READ       = 1;           // Open existing msg for reading, fail if none
  OPENMSG_WRITE      = 2;           // Open existing msg for writing, fail if none
  OPENMSG_READWRITE  = 3;           // Open existing msg for reading and writing, fail if none

  // FmaOpenMsg() flags

const
  FOM_KEEPUMSG       = $00000001;  // Keep unique message id in smh.umsg

  // FmaOpenMsg() special message number values

const
  TMsgNumber_FAIL          = TMsgNumber( 0);    // Returned when FmaGetCurMsg fail
  TMsgNumber_CUR           = TMsgNumber(-1);    // Open current message, fail if does not exist
  TMsgNumber_PREV          = TMsgNumber(-2);    // Open previous message, fail if does not exist
  TMsgNumber_NEXT          = TMsgNumber(-3);    // Open next message, fail if does not exist
  TMsgNumber_CURORPREV     = TMsgNumber(-4);    // Open current or previous if current does not exist
  TMsgNumber_CURORNEXT     = TMsgNumber(-5);    // Open current or next if current does not exist

  // FmaUidToMsgn() flags

const
  UMSG_EXACT        =  $0001;      // Return exact message number
  UMSG_NEXT         =  $0002;      // Return next message number
  UMSG_PREV         =  $0004;      // Return previous message number

  // FmaLoadMsg() flags

const
  FLM_NOTEXT        =  $0001;      // Don't load message text

  // FmaSaveMsg() flags

const
  FSM_NOTEXT        =  $0001;      // Don't save message text
  FSM_FORCEINTL     =  $0002;      // Always add ^aINTL kludge
  FSM_AUTODOMAIN    =  $0010;      // Add ^aDOMAIN if both domains are known
  FSM_AUTOMSGID     =  $0020;      // Add ^aMSGID automatically
  FSM_AUTOPID       =  $0040;      // Add ^aPID automatically

  // FmaReadMsg() special values

const
  FRM_GETCTRLCOPY = 0;              // *pcchCtrl: Allocate a copy of msg control info
  FRM_GETCTRLPTR  = ULONG(-1);     // *pcchCtrl: Return a readonly pointer to msg control info
  FRM_GETALLTEXT  = ULONG(-1);     // *pcchText: read entire message text

  // FmaGet/SetLRInfo structure

type
  LRINFO = packed record (* lri *)
    umsgLast: UMSG;                   // Last read message uid
    umsgHigh: UMSG;                   // Highest read message uid
  end;
  PLRINFO = ^LRINFO;

  // Function prototypes

{$IFNDEF FMA_STATIC_LINKAGE}
function FmaOpenArea(pszPath: PAnsiChar; _type: UINT; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle; stdcall;

function FmaCloseArea(harea: TAreaHandle): BOOL; stdcall;
function FmaLockArea(harea: TAreaHandle; bLock: BOOL): BOOL; stdcall;

function FmaValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL; stdcall;
function FmaCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL; stdcall;
function FmaEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL; stdcall;

function FmaDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function FmaSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function FmaGetCurMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function FmaGetFirstMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function FmaGetHighMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function FmaGetMsgCount(harea: TAreaHandle): LONG; stdcall;

function FmaMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG; stdcall;
function FmaUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber; stdcall;

function FmaOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle; stdcall;
function FmaCloseMsg(hmsg: TMsgHandle): BOOL; stdcall;

function FmaLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL; stdcall;
function FmaSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL; stdcall;

function FmaReadMsg(hmsg: TMsgHandle; pmsgh: Pointer; var ppchCtrl: Pointer; pcchCtrl: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL; stdcall;

function FmaWriteMsg(hmsg: TMsgHandle; pmsgh: Pointer; pchCtrl: Pointer; cchCtrl: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL; stdcall;

function FmaGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL; stdcall;
function FmaGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL; stdcall;

function FmaGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL; stdcall;
function FmaSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PAnsiChar): BOOL; stdcall;
{$ENDIF}
/////////////////////////////////////////////////////////////////////////////
// Api management declarations
/////////////////////////////////////////////////////////////////////////////

  // Message base types

const
  MBTYPE_NONE      = 0;             // No message base
  MBTYPE_FTS       = 1;             // FTS-001 message base
  MBTYPE_SQU       = 2;             // Squish message base
  MBTYPE_JAM       = 3;             // JAM message base
  MBTYPE_PKT       = 4;             // Pkt message base
  MBTYPE_USER      = 5;             // User message base id
  MBTYPE_MAXNO     = 10;            // Max number of message bases

  // Message base api structure

type
  TFmaOpenArea = function (pszPath: PAnsiChar; _type: UINT; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle stdcall;

  TFmaCloseArea = function (harea: TAreaHandle): BOOL stdcall;
  TFmaLockArea = function (harea: TAreaHandle; bLock: BOOL): BOOL stdcall;

  TFmaValidateMsg = function (harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL stdcall;
  TFmaCreateMsgMap = function (harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL stdcall;
  TFmaEnumMsgs = function (harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL stdcall;

  TFmaDeleteMsg = function (harea: TAreaHandle; msgn: TMsgNumber): BOOL stdcall;
  TFmaSetCurMsg = function (harea: TAreaHandle; msgn: TMsgNumber): BOOL stdcall;
  TFmaGetCurMsg = function (harea: TAreaHandle): TMsgNumber stdcall;
  TFmaGetFirstMsg = function (harea: TAreaHandle): TMsgNumber stdcall;
  TFmaGetHighMsg = function (harea: TAreaHandle): TMsgNumber stdcall;
  TFmaGetMsgCount = function (harea: TAreaHandle): LONG stdcall;

  TFmaMsgnToUid = function (harea: TAreaHandle; msgn: TMsgNumber): UMSG stdcall;
  TFmaUidToMsgn = function (harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber stdcall;

  TFmaOpenMsg = function (harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle stdcall;
  TFmaCloseMsg = function (hmsg: TMsgHandle): BOOL stdcall;

  TFmaLoadMsg = function (hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL stdcall;
  TFmaSaveMsg = function (hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL stdcall;

  TFmaReadMsg = function (hmsg: TMsgHandle; pmsgh: Pointer; var ppchCtrl: Pointer; pcchCtrl: PULONG;
    var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL stdcall;

  TFmaWriteMsg = function (hmsg: TMsgHandle; pmsgh: Pointer; pchCtrl: Pointer; cchCtrl: ULONG;
    pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL stdcall;

  TFmaGetMsgCtrlLen = function (hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL stdcall;
  TFmaGetMsgTextLen = function (hmsg: TMsgHandle; var pcchText: ULONG): BOOL stdcall;

  TFmaGetLRInfo = function (harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL stdcall;
  TFmaSetLRInfo = function (harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PAnsiChar): BOOL stdcall;

type
  API = packed record (* api *)
    cb: UINT;                          // Size of this structure
    pfnOpenArea: TFmaOpenArea;
    pfnCloseArea: TFmaCloseArea;
    pfnLockArea: TFmaLockArea;
    pfnValidateMsg: TFmaValidateMsg;
    pfnCreateMsgMap: TFmaCreateMsgMap;
    pfnEnumMsgs: TFmaEnumMsgs;
    pfnDeleteMsg: TFmaDeleteMsg;
    pfnSetCurMsg: TFmaSetCurMsg;
    pfnGetCurMsg: TFmaGetCurMsg;
    pfnGetFirstMsg: TFmaGetFirstMsg;
    pfnGetHighMsg: TFmaGetHighMsg;
    pfnGetMsgCount: TFmaGetMsgCount;
    pfnMsgnToUid: TFmaMsgnToUid;
    pfnUidToMsgn: TFmaUidToMsgn;
    pfnOpenMsg: TFmaOpenMsg;
    pfnCloseMsg: TFmaCloseMsg;
    pfnLoadMsg: TFmaLoadMsg;
    pfnSaveMsg: TFmaSaveMsg;
    pfnReadMsg: TFmaReadMsg;
    pfnWriteMsg: TFmaWriteMsg;
    pfnGetMsgCtrlLen: TFmaGetMsgCtrlLen;
    pfnGetMsgTextLen: TFmaGetMsgTextLen;
    pfnGetLRInfo: TFmaGetLRInfo;
    pfnSetLRInfo: TFmaSetLRInfo;
  end;
  PAPI = ^API;

type
  TFmaGetApi = function: PAPI stdcall;

  // Function prototypes

{$IFNDEF FMA_STATIC_LINKAGE}
function FmaRegisterApi(_type: UINT; pfnGetApi: TFmaGetApi): BOOL; stdcall;
{$ENDIF}
/////////////////////////////////////////////////////////////////////////////
// Miscellaneous declarations
/////////////////////////////////////////////////////////////////////////////

  // Define newline string

const
  CRLF = #13#10;

  // Check if pointer points to newline string

function ISCRLF(pch: PAnsiChar): Boolean;

  // Check for linear white space character

function ISLWSP(ch: Char): Boolean;

  // Check for end-of-line character

function ISEOL(ch: Char): Boolean;

  // Version info structure

type
  FMAVER = record (* fmaVer *)
    verMajor: UINT;                   // Major version number
    verMinor: UINT;                   // Minor version number
    verBuild: UINT;                   // Build number
    achName: array[0..64] of AnsiChar;// Build name
  end;
  PFMAVER = ^FMAVER;

  // Initialization structure

type
  TFmaMemAlloc = function (cb: Integer; bZero: BOOL): Pointer stdcall;
  TFmaMemRealloc = function (p: Pointer; cb: Integer): Pointer stdcall;
  TFmaMemFree = procedure (p: Pointer) stdcall;

type
  FMAINIT = packed record (* fmaInit *)
  	pfnMemAlloc: TFmaMemAlloc;
  	pfnMemRealloc: TFmaMemRealloc;
  	pfnMemFree: TFmaMemFree;
  end;
  PFMAINIT = ^FMAINIT;

  // Function prototypes

{$IFNDEF FMA_STATIC_LINKAGE}
function FmaInitialize(pfmaInit: PFMAINIT): BOOL; stdcall;
function FmaTerminate: BOOL; stdcall;
function FmaGetVersion(var pfmaVer: FMAVER): BOOL; stdcall;
{$ENDIF}
/////////////////////////////////////////////////////////////////////////////
// Error codes returned by FmaGetLastError() 
/////////////////////////////////////////////////////////////////////////////

  // Common error codes

const
  FMAERROR_NONE                =   0;
  FMAERROR_INTERNAL_ERROR      =   1;
  FMAERROR_NOT_SUPPORTED       =   2;
  FMAERROR_INVALID_PARAM       =   3;
  FMAERROR_INVALID_DATA        =   4;
  FMAERROR_INVALID_SIZE        =   5;
  FMAERROR_NOT_ENOUGH_MEMORY   =   6;
  FMAERROR_NOT_FOUND           =   7;
  FMAERROR_ALREADY_EXISTS      =   8;
  FMAERROR_STILL_OPEN          =   9;
  FMAERROR_CANNOT_CREATE       =   10;
  FMAERROR_CANNOT_OPEN         =   11;
  FMAERROR_CANNOT_SEEK         =   12;
  FMAERROR_CANNOT_READ         =   13;
  FMAERROR_CANNOT_WRITE        =   14;
  FMAERROR_CANNOT_DELETE       =   15;
  FMAERROR_CANNOT_LOCK         =   16;
  FMAERROR_CANNOT_UNLOCK       =   17;
  FMAERROR_NO_DISK_SPACE       =   18;
  FMAERROR_NOT_LOCKED          =   19;
  FMAERROR_NOT_EXCLUSIVE       =   20;
  FMAERROR_NOT_CACHING         =   21;
  FMAERROR_FRAME_IN_USE        =   22;
  FMAERROR_INVALID_FRAME       =   23;
  FMAERROR_INVALID_INDEX       =   24;
  FMAERROR_ACCESS_DENIED       =   25;
  FMAERROR_MISSING_FILES       =   26;
  FMAERROR_INVALID_ADDRESS     =   27;
  FMAERROR_INVALID_KLUDGE      =   28;
  FMAERROR_INVALID_USERID      =   29;
  FMAERROR_NOT_REGISTERED      =   30;
  FMAERROR_DATA_SIZE_LIMIT     =   31;

  FMAERROR_CUSTOM_ERRORS_BASE  =   100;

  // Function prototypes

{$IFNDEF FMA_STATIC_LINKAGE}
function FmaSetLastError(code: TFmaError): TFmaError; stdcall;
function FmaGetLastError: TFmaError; stdcall;
{$ENDIF}

implementation

function PFMPPSZ(pfmp: PFMP): PAnsiChar;
begin
  Result := PAnsiChar(pfmp) + sizeof(FMP);
end;

function PFMPDATA(pfmp: PFMP): PBYTE;
begin
  Result := PBYTE(PAnsiChar(pfmp) + sizeof(FMP));
end;

function PFMPLONG(pfmp: PFMP): PLONG;
begin
  Result := PLONG(PAnsiChar(pfmp) + sizeof(FMP));
end;

function PFMPDTTM(pfmp: PFMP): PDTTM;
begin
  Result := PDTTM(PAnsiChar(pfmp) + sizeof(FMP));
end;

function FMP_FLG(id: DWORD): DWORD;
begin
  Result := id and FMP_FLGMASK;
end;

function FMP_POS(id: DWORD): DWORD;
begin
  Result := (id and FMP_POSMASK) shr 16;
end;

function FMP_TAG(id: DWORD): DWORD;
begin
  Result := id and FMP_TAGMASK;
end;

function _FMP(tag, pos, flg: DWORD): DWORD;
begin
  Result :=
    (tag and FMP_TAGMASK) or
    ((pos shl 16) and FMP_POSMASK) or
    (flg and FMP_FLGMASK);
end;

  // Check if pointer points to newline string

function ISCRLF(pch: PAnsiChar): Boolean;
begin
  Result := ((pch[0] = #13) and (pch[1] = #10));
end;

  // Check for linear white space character

function ISLWSP(ch: Char): Boolean;
begin
  Result := ((ch = ' ') or (ch = #9));
end;

  // Check for end-of-line character

function ISEOL(ch: Char): Boolean;
begin
  Result := ((ch = #13) or (ch = #10));
end;

{$IFNDEF FMA_STATIC_LINKAGE}
function FmaCreateMsg(cbExtra: UINT): PFMM;
  external FmaDll name 'FmaCreateMsg';
function FmaDestroyMsg(pfmm: PFMM): BOOL; 
  external FmaDll name 'FmaDestroyMsg';
function FmaDupeMsg(pfmm: PFMM): PFMM; 
  external FmaDll name 'FmaDupeMsg';
function FmaDumpMsg(pfmm: PFMM): BOOL; 
  external FmaDll name 'FmaDumpMsg';

function FmaGetFmp(pfmm: PFMM; id: ULONG; ifmp: UINT): PFMP;
  external FmaDll name 'FmaGetFmp';
function FmaDelFmp(pfmm: PFMM; id: ULONG; ifmp: UINT): BOOL;
  external FmaDll name 'FmaDelFmp';
function FmaDumpFmp(pfmp: PFMP): BOOL;
  external FmaDll name 'FmaDumpFmp';
function FmaAddFmp(pfmm: PFMM; id: ULONG; pbData: Pointer; cbData: ULONG): PFMP;
  external FmaDll name 'FmaAddFmp';
function FmaAddFmpFormatV(pfmm: PFMM; id: ULONG; pszFormat: PAnsiChar; parg: Tva_list): PFMP;
  external FmaDll name 'FmaAddFmpFormatV';
function FmaGetFmpTextLen(pfmp: PFMP): ULONG;
  external FmaDll name 'FmaGetFmpTextLen';

function FmaOpenArea(pszPath: PAnsiChar; _type: UINT; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle;
  external FmaDll name 'FmaOpenArea';

function FmaCloseArea(harea: TAreaHandle): BOOL;
  external FmaDll name 'FmaCloseArea';
function FmaLockArea(harea: TAreaHandle; bLock: BOOL): BOOL;
  external FmaDll name 'FmaLockArea';

function FmaValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL;
  external FmaDll name 'FmaValidateMsg';
function FmaCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL; 
  external FmaDll name 'FmaCreateMsgMap';
function FmaEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL;
  external FmaDll name 'FmaEnumMsgs';

function FmaDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL;
  external FmaDll name 'FmaDeleteMsg';
function FmaSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; 
  external FmaDll name 'FmaSetCurMsg';
function FmaGetCurMsg(harea: TAreaHandle): TMsgNumber; 
  external FmaDll name 'FmaGetCurMsg';
function FmaGetFirstMsg(harea: TAreaHandle): TMsgNumber; 
  external FmaDll name 'FmaGetFirstMsg';
function FmaGetHighMsg(harea: TAreaHandle): TMsgNumber; 
  external FmaDll name 'FmaGetHighMsg';
function FmaGetMsgCount(harea: TAreaHandle): LONG; 
  external FmaDll name 'FmaGetMsgCount';

function FmaMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG; 
  external FmaDll name 'FmaMsgnToUid';
function FmaUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber; 
  external FmaDll name 'FmaUidToMsgn';

function FmaOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle; 
  external FmaDll name 'FmaOpenMsg';
function FmaCloseMsg(hmsg: TMsgHandle): BOOL;
  external FmaDll name 'FmaCloseMsg';

function FmaLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL; 
  external FmaDll name 'FmaLoadMsg';
function FmaSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL; 
  external FmaDll name 'FmaSaveMsg';

function FmaReadMsg(hmsg: TMsgHandle; pmsgh: Pointer; var ppchCtrl: Pointer; pcchCtrl: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL; 
  external FmaDll name 'FmaReadMsg';

function FmaWriteMsg(hmsg: TMsgHandle; pmsgh: Pointer; pchCtrl: Pointer; cchCtrl: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL; 
  external FmaDll name 'FmaWriteMsg';

function FmaGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL; 
  external FmaDll name 'FmaGetMsgCtrlLen';
function FmaGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL; 
  external FmaDll name 'FmaGetMsgTextLen';

function FmaGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL;
  external FmaDll name 'FmaGetLRInfo';
function FmaSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PAnsiChar): BOOL;
  external FmaDll name 'FmaSetLRInfo';

function FmaRegisterApi(_type: UINT; pfnGetApi: TFmaGetApi): BOOL;
  external FmaDll name 'FmaRegisterApi';

function FmaInitialize(pfmaInit: PFMAINIT): BOOL; 
  external FmaDll name 'FmaInitialize';
function FmaTerminate: BOOL; 
  external FmaDll name 'FmaTerminate';
function FmaGetVersion(var pfmaVer: FMAVER): BOOL; 
  external FmaDll name 'FmaGetVersion';

function FmaSetLastError(code: TFmaError): TFmaError;
  external FmaDll name 'FmaSetLastError';
function FmaGetLastError: TFmaError; 
  external FmaDll name 'FmaGetLastError';
{$ENDIF}

end.
