unit FmaCtrl;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Control info routines header file
//
// FmaCtrl.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 26/Sep/1999
// Updated: 17/Feb/2000
//
// History: 26/Sep/1999 kvitekp - Initially created
//          17/Feb/2000 akzhan  - Initially converted to Delphi
//
/////////////////////////////////////////////////////////////////////////////

{$I FmxDef.inc}
interface

uses
  Windows, FmaDef, FmaApi, FmaUti;

/////////////////////////////////////////////////////////////////////////////
// Control info parser declarations
/////////////////////////////////////////////////////////////////////////////

  // End-of-kludge macro
function EOKL(ch: AnsiChar): Boolean;

  // Attribute mapping table entry macro
  //   isn't used - #define ATTR_(attr, attrNative, flag) { attr, attrNative, flag },

  // Attribute mapping table structure

type
  ATTRMAP = packed record
  	attr: ULONG;               // Fidonet mail attribute (MA_)
	  attrNative: ULONG;         // Native message attribute
  	pszFlag: PAnsiChar;        // Matching ^aFLAGS kludge flag
  end;
  PATTRMAP = ^ATTRMAP;

  // Message header mapping table entry types

const
  MHDR_UL32     =  1;       // Unsigned long 32 bit value
  MHDR_US16     =  2;       // Unsigned short 16 bit value
  MHDR_DTTM     =  3;       // Date/time info 32-bit, DTTM format
  MHDR_TIME     =  4;       // Date/time info 32-bit, time format
  MHDR_ASCZ     =  5;       // Asciiz string, bSave == cchMax
  MHDR_ADDR     =  6;       // Binary netword address (NETADDR)
  MHDR_LAST     =  6;       // Use this for defining private types

type
  // Message header mapping table entry structure

  MHDRMAP = packed record
    offs: UINT;               // Offset into the message header
    cb: UINT;                 // Size of the message header field
    _type: UINT;              // Message header entry type (MHDR_)
    id: ULONG;                // Fidonet mail message property id (FMP_)
  end;
  PMHDRMAP = ^MHDRMAP;

  // Kludge info structure types. Note that since these
  // are also used as flags in tables, its one based

const
  KLI_FMPT     =      1;
  KLI_TOPT     =      2;
  KLI_INTL     =      3;
  KLI_DOMAIN   =      4;
  KLI_FLAGS    =      5;
  KLI_MSGID    =      6;
  KLI_REPLY    =      7;
  KLI_NOPREFIX =  $8000;

  // Kludge info structure

type
  KLI = packed record (* kli *)
	  pch: array[0..7] of PAnsiChar; // Kludge data pointer array indexed by KLI_
	  cch: array[0..7] of UINT;        // Kludge data length array indexed by KLI_
	  bSeenEomKludge: BOOL;          // TRUE is have seen any of known eom kludges
  end;
  PKLI = ^KLI;

  // Kludge mapping table entry structure

  KLMAP = packed record (* klme *)
	  id: ULONG;                // Fidonet mail property id (FMP_)
	  psz: PAnsiChar;           // Kludge name pointer
	  cch: UINT;                // Kludge name length
	  _type: UINT;              // Kludge type (KLI_)
  end;
  PKLMAP = ^KLMAP;

  // Function prototypes

{$IFNDEF FMA_STATIC_LINKAGE}
function FmaGetNetAddr(pfmm: PFMM; id: ULONG; var pnetAddr: NETADDR;
              var ppchDomain: PAnsiChar): BOOL; stdcall;

function FmaLoadMhdr(pfmm: PFMM; pmhdr: Pointer; aMhdr: PMHDRMAP; nMhdr: UINT): BOOL; stdcall;
function FmaSaveMhdr(pfmm: PFMM; pmhdr: Pointer; aMhdr: PMHDRMAP; nMhdr: UINT): BOOL; stdcall;

function FmaLoadFLAGS(pfmm: PFMM; pchData: PAnsiChar; cchData: UINT): BOOL; stdcall;

function FmaLoadKludge(pfmm: PFMM; pch: PAnsiChar; cch: UINT; var pkli: KLI): BOOL; stdcall;
function FmaSaveKludge(pfmm: PFMM; pfmp: PFMP; pdszCtrl: PDSZ; bAddCR: BOOL): BOOL; stdcall;

function FmaLoadAddrKludges(pfmm: PFMM; var pkli: PKLI; var pnetOrig: NETADDR;
                 var pnetDest: NETADDR; var pnetDefault: NETADDR;
                 bEchoMail: BOOL): BOOL; stdcall;

function FmaSaveAddrKludges(pfmm: PFMM; pdszCtrl: PDSZ; var pnetOrig: NETADDR;
                 var pnetDest: NETADDR; var pnetDefault: NETADDR;
                 bNoMhdrZoneInfo: BOOL; bNoMhdrPointInfo: BOOL;
                 fl: ULONG; bAddCR: BOOL): BOOL; stdcall;

function FmaLoadEomKludges(pfmm: PFMM; fl: ULONG): BOOL; stdcall;
function FmaSaveEomKludges(pfmm: PFMM; pdszEom: PDSZ): BOOL; stdcall;

function FmaGetFlagFromAttr(attr: ULONG): PAnsiChar; stdcall;
function FmaGetAttrFromFlag(pch: PAnsiChar; cch: UINT): ULONG; stdcall;

function FmaGetKludgeFromId(id: ULONG): PKLMAP; stdcall;
function FmaGetKludgeFromString(pch: PAnsiChar; cch: UINT): PKLMAP; stdcall;

function FmaGetEomKludgeFromId(id: ULONG): PKLMAP; stdcall;
function FmaGetEomKludgeFromString(pch: PAnsiChar; cch: UINT; bHasPrefix: BOOL): PKLMAP; stdcall;

function FmaAddAutoMSGID(pfmm: PFMM): PFMP; stdcall;
function FmaAddAutoPID(pfmm: PFMM): PFMP; stdcall;
function FmaAddAutoProp(pfmm: PFMM; fl: ULONG): BOOL; stdcall;
{$ENDIF}
implementation

function EOKL(ch: AnsiChar): Boolean;
begin
// # define EOKL(ch) ((UCHAR)(ch) <= 0x01 || (ch) == '\r')
  Result := ch in [#0, #1, #13];
end;

{$IFNDEF FMA_STATIC_LINKAGE}
function FmaGetNetAddr(pfmm: PFMM; id: ULONG; var pnetAddr: NETADDR;
              var ppchDomain: PAnsiChar): BOOL;
  external FmaDll name 'FmaGetNetAddr';

function FmaLoadMhdr(pfmm: PFMM; pmhdr: Pointer; aMhdr: PMHDRMAP; nMhdr: UINT): BOOL;
  external FmaDll name 'FmaLoadMhdr';
function FmaSaveMhdr(pfmm: PFMM; pmhdr: Pointer; aMhdr: PMHDRMAP; nMhdr: UINT): BOOL;
  external FmaDll name 'FmaSaveMhdr';

function FmaLoadFLAGS(pfmm: PFMM; pchData: PAnsiChar; cchData: UINT): BOOL;
  external FmaDll name 'FmaLoadFLAGS';

function FmaLoadKludge(pfmm: PFMM; pch: PAnsiChar; cch: UINT; var pkli: KLI): BOOL;
  external FmaDll name 'FmaLoadKludge';
function FmaSaveKludge(pfmm: PFMM; pfmp: PFMP; pdszCtrl: PDSZ; bAddCR: BOOL): BOOL;
  external FmaDll name 'FmaSaveKludge';

function FmaLoadAddrKludges(pfmm: PFMM; var pkli: PKLI; var pnetOrig: NETADDR;
                 var pnetDest: NETADDR; var pnetDefault: NETADDR;
                 bEchoMail: BOOL): BOOL;
  external FmaDll name 'FmaLoadAddrKludges';

function FmaSaveAddrKludges(pfmm: PFMM; pdszCtrl: PDSZ; var pnetOrig: NETADDR;
                 var pnetDest: NETADDR; var pnetDefault: NETADDR;
                 bNoMhdrZoneInfo: BOOL; bNoMhdrPointInfo: BOOL;
                 fl: ULONG; bAddCR: BOOL): BOOL;
  external FmaDll name 'FmaSaveAddrKludges';

function FmaLoadEomKludges(pfmm: PFMM; fl: ULONG): BOOL;
  external FmaDll name 'FmaLoadEomKludges';
function FmaSaveEomKludges(pfmm: PFMM; pdszEom: PDSZ): BOOL;
  external FmaDll name 'FmaSaveEomKludges';

function FmaGetFlagFromAttr(attr: ULONG): PAnsiChar;
  external FmaDll name 'FmaGetFlagFromAttr';
function FmaGetAttrFromFlag(pch: PAnsiChar; cch: UINT): ULONG;
  external FmaDll name 'FmaGetAttrFromFlag';

function FmaGetKludgeFromId(id: ULONG): PKLMAP;
  external FmaDll name 'FmaGetKludgeFromId';
function FmaGetKludgeFromString(pch: PAnsiChar; cch: UINT): PKLMAP;
  external FmaDll name 'FmaGetKludgeFromString';

function FmaGetEomKludgeFromId(id: ULONG): PKLMAP;
  external FmaDll name 'FmaGetEomKludgeFromId';
function FmaGetEomKludgeFromString(pch: PAnsiChar; cch: UINT; bHasPrefix: BOOL): PKLMAP;
  external FmaDll name 'FmaGetEomKludgeFromString';

function FmaAddAutoMSGID(pfmm: PFMM): PFMP;
  external FmaDll name 'FmaAddAutoMSGID';
function FmaAddAutoPID(pfmm: PFMM): PFMP;
  external FmaDll name 'FmaAddAutoPID';
function FmaAddAutoProp(pfmm: PFMM; fl: ULONG): BOOL;
  external FmaDll name 'FmaAddAutoProp';
{$ENDIF}

end.
