unit FmaDef;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Common constants and types declaration 
//
// FmaDef.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 27/Aug/1999
// Updated: 14/Oct/2002
//
// History: 17/Jan/1999 kvitekp - Initially created for IMA project.
//          01/Feb/2000 akzhan  - Convertion performed, DTTM structure
//                                was interpreted as DWORD, so
//                                conversion utilities defined too.
//          14/Feb/2001 akzhan  - NoAreaHandle, NoNetAddr and NoMsgHandle constants
//                                provided.
//                                SameNetAddr test function implemented.
//          14/0ct/2002 Plyako A- Fixed some problem with DateTime manipulation
//
/////////////////////////////////////////////////////////////////////////////

{$IFNDEF VIRTUALPASCAL}
{$WEAKPACKAGEUNIT ON}
{$ENDIF}
interface
// Set structure packing for common byte packed structures
{$I FmxDef.inc}

uses
  Windows;

const
  FmaDll = 'FmaVC6.dll';

/////////////////////////////////////////////////////////////////////////////
// Additional types declaration
/////////////////////////////////////////////////////////////////////////////

type
  // Common types
  AreaHandlePointsTo = packed record
  end;
  TAreaHandle = ^AreaHandlePointsTo;    // Message area handle
const
  NoAreaHandle = TAreaHandle(nil);

type
  MsgHandlePointsTo = packed record
  end;
  TMsgHandle = ^MsgHandlePointsTo;      // Message handle
const
  NoMsgHandle = TMsgHandle(nil);

type
  TMsgNumber = ULONG;                   // Physical message number
  UMSG = {$IFNDEF VIRTUALPASCAL} type {$ENDIF} ULONG;  // Unique message id
  TMsgId = UMSG;
const
  NoMsgId = TMsgId(0); 

type
  TFmaFile = {$IFNDEF VIRTUALPASCAL} type {$ENDIF} Integer;   // hfile

  USHORT = Word;
  LONG = Longint;
  PPointer = ^Pointer;
  PLONG = ^LONG;
  PPAnsiChar = ^PAnsiChar;
  PExtended = ^Extended;
  PSZ = PAnsiChar;
  PVOID = pointer;
  PCH = PAnsiChar;

type
  Tva_list = Pointer;
  AUMSG = packed array[0..0] of UMSG;
  PUMSG = ^UMSG;
  PPUMSG = ^PUMSG;

  // Network address structure
  NETADDR = packed record (* net *)
     zone: USHORT;                       // Zone number
     net: USHORT;                        // Network number
     node: USHORT;                       // Node number
     point: USHORT;                      // Point number
  end;
//  NETADDR = _NETADDR;
  PNETADDR = ^NETADDR;
  TFmaNetAddr = NETADDR;

const
  NoNetAddr: TFmaNetAddr = (zone: USHORT(-1); net: USHORT(-1); node: USHORT(-1); point: USHORT(-1));

function SameNetAddr(A: TFmaNetAddr; B: TFmaNetAddr): Boolean;

type
  _DTTM = packed record (* dttm *)
    dt: Word;
    tm: Word;
  end;
  // Date and time structure
  // now interpreted as DWORD
  P_DTTM = ^_DTTM;
  DTTM = {$IFNDEF VIRTUALPASCAL} type {$ENDIF} DWORD;
  PDTTM = ^DTTM;
  TDTTM = DTTM;

procedure DecodeDTTM(dttm: DTTM; var Sec: Word; var Min: Word; var Hour: Word; var Day: Word; var Month: Word; var Year: Word);
function EncodeDTTM(Sec: Word; Min: Word; Hour: Word; Day: Word; Month: Word; Year: Word): DTTM;

type
  TFmaError = {$IFNDEF VIRTUALPASCAL} type {$ENDIF} UINT;

implementation

function SameNetAddr(A: TFmaNetAddr; B: TFmaNetAddr): Boolean;
begin
  with A do
  begin
    Result := (zone = B.zone) and (net = B.net) and (node = B.node) and (point = B.point);
  end;
end;

procedure DecodeDT(dt: Word; var Day: Word; var Month: Word; var Year: Word);
begin
  Day := dt and $001F;
  dt := dt shr 5;
  Month := dt and $000F;
  dt := dt shr 4;
  Year := 1980 + dt;
end;

function EncodeDT(Day: Word; Month: Word; Year: Word): Word;
begin
  Result := (Day and $001F) or ((Month and $000F) shl 5) or (((Year - 1980) and $007F) shl 9);
end;

procedure DecodeTM(tm: Word; var Hour: Word; var Min: Word; var Sec: Word);
begin
  Sec := (tm and $001F) shl 1;
  tm := tm shr 5;
  Min := tm and $003F;
  tm := tm shr 6;
  Hour := tm;
end;

function EncodeTM(Hour: Word; Min: Word; Sec: Word): Word;
begin
  Result := ((Sec shr 1) and $001F) or ((Min and $003F) shl 5) or ((Hour and $001F) shl 11);
end;

procedure DecodeDTTM(dttm: DTTM; var Sec: Word; var Min: Word; var Hour: Word; var Day: Word; var Month: Word; var Year: Word);
var
  dt: Word;
  tm: Word;
begin
  dt := LOWORD(dttm);
  tm := HIWORD(dttm);
  DecodeDT(dt, Day, Month, Year);
  DecodeTM(tm, Hour, Min, Sec);
end;

function EncodeDTTM(Sec: Word; Min: Word; Hour: Word; Day: Word; Month: Word; Year: Word): DTTM;
begin
  Result := EncodeDT(Day, Month, Year) or (DWORD(EncodeTM(Hour, Min, Sec)) shl 16);
end;

end.
