unit FmaSys;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: System dependant routines header file
//
// FmaSys.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 27/Aug/1999
// Updated: 09/Feb/2000
//
// History: 17/Jan/1999 kvitekp - Initially created for IMA project,
//
//          01/Feb/2000 akzhan  - Initially converted for Delphi.
//                                TFmaMemAlloc, TFmaMemRealloc, TFmaMemFree
//                                types defined and used in FmaApi.FMAINIT
//                                record type
//          09/Feb/2000 kvitekp - Added FmaGet/SetFilePos
//          09/Feb/2000 akzhan  - Added FmaGet/SetFilePos
//
/////////////////////////////////////////////////////////////////////////////

{$I FmxDef.inc}
interface

uses
  Windows, FmaDef;

/////////////////////////////////////////////////////////////////////////////
// Memory management declarations
/////////////////////////////////////////////////////////////////////////////

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaMemAlloc(cb: Integer; bZero: BOOL): Pointer; stdcall;
function FmaMemRealloc(p: Pointer; cb: Integer): Pointer; stdcall;
procedure FmaMemFree(p: Pointer); stdcall;
{$ENDIF}
  // Flags used by MemAlloc

const
  MA_CLEAR = true;

type
  TFmaMemAlloc = function (cb: Integer; bZero: BOOL): Pointer stdcall;
  TFmaMemRealloc = function (p: Pointer; cb: Integer): Pointer stdcall;
  TFmaMemFree = procedure (p: Pointer) stdcall;

  // Map functions to common names redirected through global thunk

{
// these macros isn't converted to Object Pascal - obsolete
#define MemAlloc    fmaInit.pfnMemAlloc
#define MemRealloc  fmaInit.pfnMemRealloc
#define MemFree     fmaInit.pfnMemFree
}

/////////////////////////////////////////////////////////////////////////////
// File management declarations
/////////////////////////////////////////////////////////////////////////////

  // FmaOpenFile() open flags

const
  FOPENFILE_READONLY =   $0001;        // Open for read only access
  FOPENFILE_WRITEONLY =  $0002;        // Open for write only access
  FOPENFILE_READWRITE =  $0004;        // Open for read/write access
  FOPENFILE_CREATE =     $0010;        // Create if does not exist
  FOPENFILE_TRUNCATE =   $0020;        // Truncate if does exists
  FOPENFILE_DENYNONE =   $0100;        // Deny none other opens
  FOPENFILE_DENYALL =    $0200;        // Deny all other opens

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaOpenFile(pszFile: PChar; fOpen: UINT; var phfile: TFmaFile): BOOL; stdcall;
function FmaCloseFile(var phfile: TFmaFile): BOOL; stdcall;
function FmaReadFile(hfile: TFmaFile; offs: LONG; pb: Pointer; cb: ULONG; var pcbRead: ULONG): BOOL; stdcall;
function FmaWriteFile(hfile: TFmaFile; offs: LONG; pb: Pointer; cb: ULONG; var pcbWritten: ULONG): BOOL; stdcall;

function FmaGetFilePos(hfile: TFmaFile): LONG;
function FmaSetFilePos(hfile: TFmaFile; offs: LONG): LONG;
function FmaGetFileSize(hfile: TFmaFile): LONG; stdcall;
function FmaSetFileSize(hfile: TFmaFile; cbFile: ULONG): BOOL; stdcall;
function FmaLock(hfile: TFmaFile; offset: LONG; cb: LONG; bLock: BOOL): BOOL; stdcall;
function FmaDeleteFile(pszFile: PChar): BOOL; stdcall;
{$ENDIF}
  // FmaEnumFiles() callback function prototype
type
  PFNENUMFILES = function(pszFile: PChar; cbFile: ULONG; attr: ULONG; pvCallbackData: Pointer): BOOL stdcall;

  // FmaEnumFiles() flags
const
  FENUMFILES_FULLPATH = $00000001;

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaEnumFiles(pszPath: PChar; pszMask: PChar; pfnCallback: PFNENUMFILES;
  pvCallbackData: Pointer; fl: ULONG): BOOL; stdcall;

function FmaFileExists(pszPath: PChar): BOOL; stdcall;
function FmaDirExists(pszPath: PChar): BOOL; stdcall;
function FmaMakeDir(pszPath: PChar): BOOL; stdcall;
{$ENDIF}
/////////////////////////////////////////////////////////////////////////////
// Miscellaneous declarations
/////////////////////////////////////////////////////////////////////////////

  // Flags used by FmaGetOSType() and FmaGiveUpTimeSlice()

const
  OSTYPE_DOS =             $0001;          // Plain DOS
  OSTYPE_WINDOWS =         $0002;          // Microsoft Windows DosBox
  OSTYPE_WINNT =           $0003;          // Microsoft Windows NT DosBox
  OSTYPE_DESQVIEW =        $0004;          // QuarterDesk DesqView
  OSTYPE_OS2DOS =          $0005;          // IBM OS/2 DosBox
  OSTYPE_OS2 =             $0006;          // Native IBM OS/2
  OSTYPE_W32 =             $0007;          // Native Win32
  OSTYPE_MACOS =           $0008;          // Native Mac OS
  OSTYPE_MASK =            $000F;          // OS type mask
  OSTYPE_PROTMODE =        $8000;          // Protected mode

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
procedure FmaInterlockedIncrement(var plv: LONG); stdcall;
procedure FmaInterlockedDecrement(var plv: LONG); stdcall;
function FmaGetLocalDttm(var pdttm: DTTM): BOOL; stdcall;
function FmaGetLocalTime: ULONG; stdcall;
procedure FmaSleep(milliseconds: ULONG); stdcall;
function FmaGetMsgIdStamp: ULONG; stdcall;
function FmaQueryOSType: UINT; stdcall;
procedure FmaGiveOutTimeSlice; stdcall;
{$ENDIF}
implementation
{$IFNDEF FMA_STATIC_LINKAGE}
function FmaMemAlloc(cb: Integer; bZero: BOOL): Pointer;
  external FmaDll name 'FmaMemAlloc';
function FmaMemRealloc(p: Pointer; cb: Integer): Pointer;
  external FmaDll name 'FmaMemRealloc';
procedure FmaMemFree(p: Pointer);
  external FmaDll name 'FmaMemFree';

function FmaOpenFile(pszFile: PChar; fOpen: UINT; var phfile: TFmaFile): BOOL;
  external FmaDll name 'FmaOpenFile';
function FmaCloseFile(var phfile: TFmaFile): BOOL;
  external FmaDll name 'FmaCloseFile';
function FmaReadFile(hfile: TFmaFile; offs: LONG; pb: Pointer; cb: ULONG; var pcbRead: ULONG): BOOL;
  external FmaDll name 'FmaReadFile';
function FmaWriteFile(hfile: TFmaFile; offs: LONG; pb: Pointer; cb: ULONG; var pcbWritten: ULONG): BOOL;
  external FmaDll name 'FmaWriteFile';

function FmaGetFilePos(hfile: TFmaFile): LONG;
  external FmaDll name 'FmaGetFilePos';
function FmaSetFilePos(hfile: TFmaFile; offs: LONG): LONG;
  external FmaDll name 'FmaSetFilePos';
function FmaGetFileSize(hfile: TFmaFile): LONG;
  external FmaDll name 'FmaGetFileSize';
function FmaSetFileSize(hfile: TFmaFile; cbFile: ULONG): BOOL;
  external FmaDll name 'FmaSetFileSize';
function FmaLock(hfile: TFmaFile; offset: LONG; cb: LONG; bLock: BOOL): BOOL;
  external FmaDll name 'FmaLock';
function FmaDeleteFile(pszFile: PChar): BOOL;
  external FmaDll name 'FmaDeleteFile';

function FmaEnumFiles(pszPath: PChar; pszMask: PChar; pfnCallback: PFNENUMFILES;
  pvCallbackData: Pointer; fl: ULONG): BOOL;
  external FmaDll name 'FmaEnumFiles';

function FmaFileExists(pszPath: PChar): BOOL;
  external FmaDll name 'FmaFileExists';
function FmaDirExists(pszPath: PChar): BOOL;
  external FmaDll name 'FmaDirExists';
function FmaMakeDir(pszPath: PChar): BOOL;
  external FmaDll name 'FmaMakeDir';

procedure FmaInterlockedIncrement(var plv: LONG);
  external FmaDll name 'FmaInterlockedIncrement';
procedure FmaInterlockedDecrement(var plv: LONG);
  external FmaDll name 'FmaInterlockedDecrement';
function FmaGetLocalDttm(var pdttm: DTTM): BOOL;
  external FmaDll name 'FmaGetLocalDttm';
function FmaGetLocalTime: ULONG;
  external FmaDll name 'FmaGetLocalTime';
procedure FmaSleep(milliseconds: ULONG);
  external FmaDll name 'FmaSleep';
function FmaGetMsgIdStamp: ULONG;
  external FmaDll name 'FmaGetMsgIdStamp';
function FmaQueryOSType: UINT;
  external FmaDll name 'FmaQueryOSType';
procedure FmaGiveOutTimeSlice;
  external FmaDll name 'FmaGiveOutTimeSlice';
{$ENDIF}
end.
