program fmatst;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Testing app
//
// fmatst.dpr (Test.c)
//
// Copyright (c) 1999 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// Created: 27/Aug/1999
// Updated: 27/Aug/1999
//
// History: 27/Aug/1999 kvitekp - Initially created.
//
/////////////////////////////////////////////////////////////////////////////
{$IFNDEF VIRTUALPASCAL}
{$APPTYPE CONSOLE}
{$ENDIF}

{$I FmxDef.inc}

uses
  Windows,
  SysUtils,
  FmaDef,
  FmaSys,
  FmxDef,
  FmaApi,
  FmaUti,
  SquApi,
  JamApi,
  FtsApi,
  PktApi,
  FmxSqu,
  FmxPkt,
  FmxJam,
  FmxFts,
  FmxApi 
  {$IFDEF FMA_STATIC_LINKAGE}, FmaObj {$ENDIF};

/////////////////////////////////////////////////////////////////////////////
// M o d u l e   s p e c i f i c   d e c l a r a t i o n s                 //
/////////////////////////////////////////////////////////////////////////////

const
  netAddr: TFmxNetAddr = (zone: 2; net: 5040; node: 55; point: 0);

/////////////////////////////////////////////////////////////////////////////
// M o d u l e   l o c a l   r o u t i n e s                               //
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// This routine initializes the app
//

function DoInitApp: Boolean;
var
  _type: UINT;
  procedure ReportRegError;
  begin
    WriteLn('FmaRegisterApi() failed, type=', _type, ', code =', FmaGetLastError);
	  FmaTerminate;
  end;
begin
  Result := False;
	// Initialize FMA and check if ok

	if not FmaInitialize(nil) then
  begin
    WriteLn('FmaInitialize() failed, code=', FmaGetLastError);
    Exit;
	end;
	// Register all core message base apis

{
  _type := MBTYPE_FTS;
	if not FmaRegisterApi(_type, FtsGetApi) then
  begin
    ReportRegError;
    Exit;
  end;
}
  _type := MBTYPE_SQU;
	if not FmaRegisterApi(_type, SquGetApi) then
  begin
    ReportRegError;
    Exit;
  end;
{
  _type := MBTYPE_JAM;
	if not FmaRegisterApi(_type, JamGetApi) then
  begin
    ReportRegError;
    Exit;
  end;
  _type := MBTYPE_PKT;
	if not FmaRegisterApi(_type, PktGetApi) then
  begin
    ReportRegError;
    Exit;
  end;
}
  Result := True;
end;

/////////////////////////////////////////////////////////////////////////////
// This routine tests open area apis
//

function DoOpenArea(pszPath: PAnsiChar; _type: UINT): Boolean;
var
	fl: ULONG;
	harea: TAreaHandle;
	hmsg: TMsgHandle;
	msgn: TMsgNumber;
begin
  fl := 0;
  Result := false;

	// Inform caller

  WriteLn(#13#10'Touching area: ', pszPath, ', type=', _type);

	// Open and close an area and check if ok
  harea := FmaOpenArea(pszPath, _type, OPENAREA_ALWAYS, fl, netAddr);
	if harea = nil then
  begin
	  WriteLn('FmaOpenArea("', pszPath, '") failed, code=', FmaGetLastError);
	  Exit;
	end;

	// Display what these return

	WriteLn('FmaGetFirstMsg()=', FmaGetFirstMsg(harea));
	WriteLn('FmaGetHighMsg()=', FmaGetHighMsg(harea));
	WriteLn('FmaGetMsgCount()=', FmaGetMsgCount(harea));
	WriteLn('FmaGetCurMsg()=', FmaGetCurMsg(harea));

	// Open the current message (this should fail after open)

	msgn := TMsgNumber_CUR;
  hmsg := FmaOpenMsg(harea, OPENMSG_READ, msgn, 0);
	if hmsg = nil then
	  WriteLn('FmaOpenMsg() failed, code=', FmaGetLastError)
 else
	  FmaCloseMsg(hmsg);

	// Loop reading next msgs

	msgn := TMsgNumber_NEXT;
	while true do
  begin
    hmsg := FmaOpenMsg(harea, OPENMSG_READ, msgn, 0);
    if hmsg = nil then break;
    WriteLn('Opened msgn=', FmaGetCurMsg(harea));
	  FmaCloseMsg(hmsg);
	end;

{
	// Loop reading prev msgs
	msgn := TMsgNumber_PREV;
	while true do
  begin
    hmsg := FmaOpenMsg(harea, OPENMSG_READ, msgn, 0);
    if hmsg = nil then break;
    WriteLn('Opened msgn=', FmaGetCurMsg(harea));
	  FmaCloseMsg(hmsg);
	end;
}

	// Close an area and check if ok

	if not FmaCloseArea(harea) then
  begin
	  WriteLn('FmaCloseArea("', pszPath, '") failed, code=', FmaGetLastError);
	  Exit;
	end;

	Result := True;
end;

/////////////////////////////////////////////////////////////////////////////
// This routine tests message saving
//

function DoSaveMsg(pszPath: PAnsiChar; _type: UINT; msgn: TMsgNumber): Boolean;
var
	aReplies: array[0..9] of UMSG;
	crc, fl: ULONG;
	psz: PAnsiChar;
	harea: TAreaHandle;
	hmsg: TMsgHandle;
	pfmm: PMsgObject;
	dttm: TDTTM;
	n: UINT;
begin
  fl := 0;
  psz := nil;
  Result := False;
	// Inform caller

	WriteLn(#13#10'Saving msgn ', msgn, ' in area: ', pszPath, '_type=', _type);

	// Open and close an area and check if ok
  harea := FmaOpenArea(pszPath, _type, OPENAREA_ALWAYS, fl, netAddr);
	if harea = nil then
  begin
	  WriteLn('FmaOpenArea("', pszPath, '") failed, code=', FmaGetLastError);
	  Exit;
	end;

	// Create new message

	msgn := 0;
  hmsg := FmaOpenMsg(harea, OPENMSG_CREATE, msgn, 0);
	if hmsg = nil then
  begin
	  WriteLn('FmaOpenMsg() failed, code=', FmaGetLastError);
  end
  else
  begin
	  // Create a message and check if ok
    pfmm := FmaCreateMsg(0);
	  if pfmm = nil then
    begin
      WriteLn('FmaCreateMsg() failed, code=', FmaGetLastError);
      Exit;
	  end;

	  // Add some message attrivutes

	  pfmm^.attr := pfmm^.attr or MA_LOCAL or MA_PRIVATE or MA_FORCEPICKUP;

	  // Add some string properties

	  FmaAddFmp(pfmm, FMP_SENDERNAME, PAnsiChar('Akzhan Abdulin'), ULONG(-1));
	  FmaAddFmp(pfmm, FMP_ORIGADDR,   PAnsiChar('2:5040/55@fidonet'), ULONG(-1));
	  FmaAddFmp(pfmm, FMP_RECEVRNAME, PAnsiChar('Pete Kvitek'), ULONG(-1));
	  FmaAddFmp(pfmm, FMP_DESTADDR,   PAnsiChar('2:5020/6@fidonet.org'), ULONG(-1));
	  FmaAddFmp(pfmm, FMP_SUBJECT,    PAnsiChar('Testing FMA/FMX for Delphi'), ULONG(-1));
//	  FmaAddFmp(pfmm, FMP_GATEADDR,   '11:22/33.44@gate.org', -1);

	  // Add some binary properties

	  FmaGetLocalDttm(dttm);
    WriteLn(DateToStr(DTTMToDateTime(dttm)));
	  FmaAddFmp(pfmm, FMP_DTTMWRITTEN, Addr(dttm), sizeof(dttm));

	  for n := 0 to High(aReplies) do aReplies[n] := 1 + n;
	  FmaAddFmp(pfmm, FMP_MSGREPLIES, Addr(aReplies[0]), sizeof(aReplies));

	  // Add some text properties

	  FmaAllocString(psz, 'Here goes a message text'#13#13'/Akzhan'#13' * Origin: FMA/FMX 1.10.93 at http://www.akzhan.midi.ru/fma/ (2:5040/55)', ULONG(-1));
	  FmaAddFmp(pfmm, FMP_MESSAGETEXT, Addr(psz), sizeof(psz));

	  // Add some kludge properties

//    FmaAddFmp(pfmm, FMP_MSGID,      "2:5020/6.1@fidonet f727a8c7", -1);
//	  FmaAddFmp(pfmm, FMP_REPLYID,    "2:5020/122.1 7ed8ce50", -1);
//	  FmaAddFmp(pfmm, FMP_FLAGS,      "XX YYY ZZZZ LOK", -1);
//    FmaAddFmp(pfmm, FMP_PID,        "FMA v??.??.???", -1);
	  FmaAddFmp(pfmm, FMP_REALNAME,   PAnsiChar('Blues player'), ULONG(-1));
//	  FmaAddFmp(pfmm, FMP_TZUTC,      "-0060", -1);
//	  FmaAddFmp(pfmm, FMP_REPLYADDR,  "pete@kvitek.com", -1);
//	  FmaAddFmp(pfmm, FMP_REPLYTO,    "2:5020/400 UUCP", -1);
	  FmaAddFmp(pfmm, FMP_FTSKLUDGE,  PAnsiChar('RFC-Distribution: fido'), ULONG(-1));

	  // Add some eom kludge properties

{
	  FmaAddFmp(pfmm, FMP_SEENBY2D,   "5020/204 238 242 362 400 604 758", -1);
	  FmaAddFmp(pfmm, FMP_SEENBY2D,   "5020/1851 1978 5021/7 5022/5 12", -1);
	  FmaAddFmp(pfmm, FMP_PATH2D,     "5020/400 238 204 52 6", -1);
	  FmaAddFmp(pfmm, FMP_EOMKLUDGE,  "Via UniMail/386 0.45b 2:5020/268, 15 Apr 1999 00:15:20 UTC +0400", -1);
	  FmaAddFmp(pfmm, FMP_EOMKLUDGE,  "Via ifmail 2:5020/52@fidonet, Thu Apr 15 1999 at 06:05 (2.11)", -1);
	  FmaAddFmp(pfmm, FMP_EOMKLUDGE,  "Via 2:5020/6@fidonet @19990415.074252.UTC+4 T-Mail 2604.NT", -1);
	  FmaAddFmp(pfmm, FMP_EOMKLUDGE,  "Recd from 2:5020/6 @19990415.082013.UTC+3", -1);
}

	  // Dump message abstraction

	  FmaDumpMsg(pfmm);

	  // Load message and check if ok

	  if not FmaSaveMsg(hmsg, pfmm, FSM_AUTOMSGID or FSM_AUTOPID, ULONG(-1)) then
  		WriteLn('FmaSaveMsg() failed, code=', FmaGetLastError);

	  // Destroy message abstraction

	  FmaDestroyMsg(pfmm);

	  // Close message

	  if not FmaCloseMsg(hmsg) then
  		WriteLn('FmaCloseMsg() failed, code=', FmaGetLastError);
  end;
	// Close an area and check if ok

	if not FmaCloseArea(harea) then
	  WriteLn('FmaCloseArea("', pszPath, '") failed, code=', FmaGetLastError)
  else
    Result := True;
end;

const 
  sFMALOG:PChar='FMA-LOG';
  sFMALOGv:PChar='*';
begin
	// Force debugging log
  SetEnvironmentVariable(sFMALOG, sFMALOGv);

	// Initialize the app and check if ok
	if not DoInitApp then Exit;

  try
  	// Basic functionality tests

  	DoOpenArea('G:\SqBase\KHABAR~4', MBTYPE_SQU);
    DoSaveMsg('G:\SqBase\RUDELPHI', MBTYPE_SQU, 544);
  finally
  	FmaTerminate;
  end;
end.
