unit uFidoRead;
/////////////////////////////////////////////////////////////////////////////
//     FMA Express
//   TMessageRead --  ""      
// ,         .
//
//   (c)   , 2002
/////////////////////////////////////////////////////////////////////////////
interface

uses
  Classes, uFidoCommon, FmxAPI, FmxCore;


type
  TMessageRead = class(TThread)
  private { Private declarations }
    // 
    FArea: TFmxCustomMessageArea;
  //  
    fName: String;
    fDate: TDateTime;
  //  
    FBaseType: TBaseType;
    FBasePath: string;
    FMsgID: string;
    FSubject: string;
    FText: string;
    FfrAddress: string;
    FtoName, FToAddress: string;
    FReplyTo: string;
    FEMail: string;
    FMsgFinded: Boolean;
    //    
    procedure FindMsg(var emi: TEnumMsgInfo; var Cancel: Boolean);
  protected
    procedure Execute; override;
  public
  {
    Echo --          
    Base --    (. uFidoCommon)
    Name --     (Sender Name)
    msgDateTime --      
  }
    constructor Create(const Echo: string; base: TBaseType; const Name: string;
                       msgDateTime: TDateTime);
    destructor Destroy; override;
    //      : ----------------------------------
    property BaseType: TBaseType read FBaseType;
    property BasePath: string read FBasePath;
    property MsgDateTime: TDateTime read fDate;
    property FromName: string read fName;
    //  ,    : ------------------------
    property Subject: string read FSubject;
    property MessageID: string read FMsgID;
    property Text: string read FText;
    property FromAddress: string read FfrAddress;
    property ToName: string read FtoName;
    property ToAddress: string read FtoAddress;
    property ReplyTo: string read FReplyTo;
    property EMail: string read FEMail;
    //   --  ,    
    property MsgFinded: Boolean read FMsgFinded;
  end;

implementation
uses SysUtils, Windows, FmaAPI, uFidoRoutine;

const
  // ""   :   1   1 
  Eps1Min1Msec = 6.94456018518519E-4;


constructor TMessageRead.Create(const Echo: string; base: TBaseType;
  const Name: string; msgDateTime: TDateTime);
begin
  inherited Create(true);
  //   :
  FBaseType := base;
  FBasePath := Echo;
  FMsgId := '';
  FSubject := '';
  FText := '';
  FFrAddress := '';
  FtoName := '';
  FReplyTo:='';
  FEMail := '';
  FMsgFinded := false;
  //   
  FArea := AreaClasses[base].Create;
  FArea.Path := Echo;
  FArea.OpenMode := aomExisting;
  //   
  fName := Name; fDate := msgDateTime;
end;

destructor TMessageRead.Destroy;
begin
  FArea.Free;
  inherited;
end;


//    
procedure TMessageRead.FindMsg(var emi: TEnumMsgInfo; var Cancel: Boolean);
var
  CurMessage: TFmxCustomAreaMessage;
  CurMessageInfo: TFmxMessageObject;
  CurMessageProp: TFmxMsgPropAccessor;
  MsgDate: TDateTime;
  FromName: string;
begin
try
  //     emi.msgn
  CurMessage := FArea.GetMessage(emi.msgn, momRead, []);
  try
   //     (    )
   CurMessageInfo := TFmxMessageObject.CreateOn(CurMessage.Load(FLM_NOTEXT));
   try
     MsgDate := 0; FromName :='';
     CurMessageProp := CurMessageInfo.Properties;
     //  ,    
     with CurMessageProp do
       begin
        if FindProperty(FMP_SENDERNAME)  then FromName := Trim(GetString);
        if FindProperty(FMP_DTTMWRITTEN) then MsgDate  := AsDateTime;
       end;
     // ,     :
     Cancel := (AnsiUpperCase(ASCIIToWin(FromName)) = AnsiUpperCase(fName)) and
               (abs(MsgDate-fDate) < Eps1Min1Msec);
     if Cancel then
     //   ,     
      with CurMessageProp do
       begin
        fDate := MsgDate;
        if FindProperty(FMP_SUBJECT)    then FSubject   := Trim(GetString);
        if FindProperty(FMP_DESTADDR)   then FtoAddress := Trim(GetString);
        if FindProperty(FMP_ORIGADDR)   then FfrAddress := Trim(GetString);
        if FindProperty(FMP_RECEVRNAME) then FtoName    := Trim(GetString);
        if FindProperty(FMP_MSGID)      then FMsgID     := Trim(GetString);

        //   ""   JAM , 
        //   FindKludge --     FMA
        if FindProperty(FMP_REPLYADDR) or FindKludge('REPLYADDR') then
           FEMail   := Trim(GetString);
        if FindProperty(FMP_REPLYTO)   or FindKludge('REPLYTO') then
           FReplyTo := Trim(GetString);
       end;
   finally
     CurMessageInfo.Free;
   end;
  finally
   CurMessage.Close;
   FreeAndNil(CurMessage);
  end;
except on E:Exception do
  MessageBox(0, PChar(SEEnumMsg+E.Message), nil,
             MB_OK or MB_ICONWARNING or MB_APPLMODAL);
end;
  //    CALLBACK    FMALastError
  //     FMA
  FmaSetLastError(FMAERROR_NONE);
end;

procedure TMessageRead.Execute;
var
  CrimeMessage: TFmxCustomAreaMessage;
  MessageList: TStringList;
begin
try
  //  
  FArea.Open;
  try
   //    
   FArea.MessageNumber := FArea.GetHighMessageNumber;

   //    " "  
   FMsgFinded := FArea.EnumMessages(FindMsg,[emfReverseOrder]);

   //    ,  ...
   if FMsgFinded then

     begin
     //     
      CrimeMessage := FArea.Get(momRead, []);
      MessageList := TStringList.Create;
      try
        CrimeMessage.Read(MessageList);
        FText := MessageList.Text;
        CrimeMessage.Close;
      finally
        MessageList.Free;
        CrimeMessage.Free;
      end;
     end;

  finally
    FArea.Close;
  end;
except on E: Exception do
  MessageBox(0, PChar(SEFindMsg+E.Message), nil,
             MB_OK or MB_ICONWARNING or MB_APPLMODAL);
end;
end;


end.
