unit FmxJam;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Express: FMX Classes that provides object APIs
//                       for FMA/Jam.
//
// FmxJam.Pas
//
// Copyright (c) 2001 by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 13/Feb/2001
// Updated: 14/Oct/2002
//
// History: 13/Feb/2001 akzhan  - Initially created for Delphi.
//          14/Feb/2001 akzhan  - Alpha 1.
//          15/Feb/2001 akzhan  - Beta (all functionality implemented).
//          14/Oct/2002 Plyako A- abstract method DoGetMessage overrided
//
/////////////////////////////////////////////////////////////////////////////

{$I FmxDef.inc}

interface

uses
  Windows, FmaDef, FmaApi, FmxApi, FmxCore;

type
  TFmxJamAreaMessage = class(TFmxCustomAreaMessage)
  protected
    function DoClose: Boolean; override;
    function DoGetControlInfoLen(var Len: ULONG): Boolean; override;
    function DoGetTextLen(var Len: ULONG): Boolean; override;

    function DoLoad(var ppfmm: PMsgObject; fl: ULONG): Boolean; override;
    function DoSave(pfmm: PMsgObject; fl: ULONG; cchTextTotal: ULONG): Boolean; override;

    function DoRead(pmsgh: Pointer; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG; var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): Boolean; override;
    function DoWrite(pmsgh: Pointer; pchCtrl: PAnsiChar; cchCtrl: ULONG; pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): Boolean; override;
  public
    constructor Create(AreaHandle: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG); override;
  end;

type
  TFmxJamMessageArea = class(TFmxCustomMessageArea)
  protected
    procedure DoOpen(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR); override;
    function DoClose: Boolean; override;
    function DoLock(LockArea: Boolean): Boolean; override;
    function DoGetLRInfo(var lri: LRINFO): Boolean; override;
    function DoSetLRInfo(const lri: LRINFO): Boolean; override;
    function DoValidateMessageNumber(MsgN: TMsgNumber; var MsgIndex: ULONG): Boolean; override;
    function DoCreateMsgMap(var paumsg: PUMSG; var pcumsg: ULONG): Boolean; override;
    function DoEnumMessages(CallProc: TEnumMsgsObjProc; Flags: TEnumMsgsFlags): Boolean; override;
    function DoGetCurrentMessage: TMsgNumber; override;
    function DoSetCurrentMessage(msgn: TMsgNumber): Boolean; override;
    function DoGetFirstMessage: TMsgNumber; override;
    function DoGetHighMessage: TMsgNumber; override;
    function DoGetMessageCount: LONG; override;
    function DoDeleteMessage(msgn: TMsgNumber): Boolean; override;
    function DoMsgIdToMsgNumber(Value: TMsgId; Mode: UINT): TMsgNumber; override;
    function DoMsgNumberToMsgId(Value: TMsgNumber): TMsgId; override;
    function DoGetMessage(msgn: TMsgNumber; Mode: UINT; Flags: ULONG): TFmxCustomAreaMessage; override;
  end;


implementation

uses
  JamApi
  {$IFDEF FMA_STATIC_LINKAGE}, FmaObj{$ENDIF};

{ TFmxJamMessageArea }

function TFmxJamMessageArea.DoClose: Boolean;
begin
  Result := JamCloseArea(Handle);
end;

function TFmxJamMessageArea.DoCreateMsgMap(var paumsg: PUMSG;
  var pcumsg: ULONG): Boolean;
begin
  Result := JamCreateMsgMap(Handle, paumsg, pcumsg);
end;

function TFmxJamMessageArea.DoDeleteMessage(msgn: TMsgNumber): Boolean;
begin
  Result := JamDeleteMsg(Handle, msgn);
end;

function TFmxJamMessageArea.DoEnumMessages(CallProc: TEnumMsgsObjProc;
  Flags: TEnumMsgsFlags): Boolean;
var
  Data: TEnumMsgsObjProcData;
begin
  Data.CallProc := CallProc;
  Result :=
    JamEnumMsgs(Handle, EnumerateMessagesToObjProc, Addr(Data), FmxMapEnumMsgsFlagsToFmaEnumMsgsFlags(Flags));
end;

function TFmxJamMessageArea.DoGetCurrentMessage: TMsgNumber;
begin
  Result := JamGetCurMsg(Handle);
end;

function TFmxJamMessageArea.DoGetFirstMessage: TMsgNumber;
begin
  Result := JamGetFirstMsg(Handle);
end;

function TFmxJamMessageArea.DoGetHighMessage: TMsgNumber;
begin
  Result := JamGetHighMsg(Handle);
end;

function TFmxJamMessageArea.DoGetLRInfo(var lri: LRINFO): Boolean;
begin
  Result := JamGetLRInfo(Handle, UserId, Addr(lri));
end;

function TFmxJamMessageArea.DoGetMessage(msgn: TMsgNumber; Mode: UINT;
  Flags: ULONG): TFmxCustomAreaMessage;
begin
  Result :=
    TFmxJamAreaMessage.Create(Handle, Mode, msgn, Flags);
end;

function TFmxJamMessageArea.DoGetMessageCount: LONG;
begin
  Result := JamGetMsgCount(Handle);
end;

function TFmxJamMessageArea.DoLock(LockArea: Boolean): Boolean;
begin
  Result := JamLockArea(Handle, LockArea);
end;

function TFmxJamMessageArea.DoMsgIdToMsgNumber(
  Value: TMsgId; Mode: UINT): TMsgNumber;
begin
  Result := JamUidToMsgn(Handle, Value, Mode);
end;

function TFmxJamMessageArea.DoMsgNumberToMsgId(
  Value: TMsgNumber): TMsgId;
begin
  Result := JamMsgnToUid(Handle, Value);
end;

procedure TFmxJamMessageArea.DoOpen(pszPath: PAnsiChar; fOpenMode: UINT;
  fl: ULONG; var pnetAddr: NETADDR);
begin
  SetHandle(
    JamOpenArea(
      pszPath,
      fOpenMode,
      fl,
      pnetAddr
      )
    );
end;

function TFmxJamMessageArea.DoSetCurrentMessage(
  msgn: TMsgNumber): Boolean;
begin
  Result := JamSetCurMsg(Handle, msgn);
end;

function TFmxJamMessageArea.DoSetLRInfo(const lri: LRINFO): Boolean;
begin
  Result := JamSetLRInfo(Handle, UserId, Addr(lri), PAnsiChar(UserName));
end;

function TFmxJamMessageArea.DoValidateMessageNumber(MsgN: TMsgNumber;
  var MsgIndex: ULONG): Boolean;
begin
  Result := JamValidateMsg(Handle, MsgN, Addr(MsgIndex));
end;

{ TFmxJamAreaMessage }

constructor TFmxJamAreaMessage.Create(AreaHandle: TAreaHandle;
  mode: UINT; msgn: TMsgNumber; fl: ULONG);
begin
  inherited CreateOn(
    JamOpenMsg(AreaHandle, mode, msgn, fl)
    );
end;

function TFmxJamAreaMessage.DoClose: Boolean;
begin
  Result := JamCloseMsg(Handle);
end;

function TFmxJamAreaMessage.DoGetControlInfoLen(
  var Len: ULONG): Boolean;
begin
  Result := JamGetMsgCtrlLen(Handle, Len);
end;

function TFmxJamAreaMessage.DoGetTextLen(var Len: ULONG): Boolean;
begin
  Result := JamGetMsgTextLen(Handle, Len);
end;

function TFmxJamAreaMessage.DoLoad(var ppfmm: PMsgObject;
  fl: ULONG): Boolean;
begin
  Result := JamLoadMsg(
    Handle,
    ppfmm,
    fl
    );
end;

function TFmxJamAreaMessage.DoRead(pmsgh: Pointer;
  var ppchCtrl: PAnsiChar; pcchCtrl: PULONG; var ppchText: PAnsiChar;
  pcchText, poffText: PULONG): Boolean;
var
  p: PJSF;
begin
  p := PJSF(ppchCtrl);
  Result := JamReadMsg(
    Handle,
    pmsgh,
    p,
    pcchCtrl,
    ppchText,
    pcchText,
    poffText
    );
  ppchCtrl := PAnsiChar(p);
end;

function TFmxJamAreaMessage.DoSave(pfmm: PMsgObject; fl,
  cchTextTotal: ULONG): Boolean;
begin
  Result := JamSaveMsg(
    Handle,
    pfmm,
    fl,
    cchTextTotal
    );
end;

function TFmxJamAreaMessage.DoWrite(pmsgh: Pointer; pchCtrl: PAnsiChar;
  cchCtrl: ULONG; pchText: PAnsiChar; cchText: ULONG; poffText: PULONG;
  cchTextTotal: ULONG): Boolean;
begin
  Result := JamWriteMsg(
    Handle,
    pmsgh,
    PJSF(pchCtrl),
    cchCtrl,
    pchText,
    cchText,
    poffText,
    cchTextTotal
    );
end;

end.
