unit FmxSys;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: FmaSys Utilities that references Delphi functionality
//
// FmxSys.Pas
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 15/Feb/2000
// Updated: 15/Feb/2000
//
// History: 15/Feb/2000 akzhan  - Initially created for Delphi convertion.
//
/////////////////////////////////////////////////////////////////////////////

interface

uses
  Windows, SysUtils, FmaDef, FmaSys;

/////////////////////////////////////////////////////////////////////////////
// File management declarations
/////////////////////////////////////////////////////////////////////////////

  // FmxOpenFile() open types

type
  TFmxFile = TFmaFile;
  TFmxFilePosition = LONG;
  TFmxFileSize = ULONG;
  TFmxFileAccessMode = (fmxFileReadOnly, fmxFileWriteOnly, fmxFileReadWrite);
  TFmxFileSharingMode = (fmxFileDenyNone, fmxFileDenyAll);
  TFmxFileOpenOption = (fmxFileCreate, fmxFileTruncate);
  TFmxFileOpenOptions = set of TFmxFileOpenOption;

  // Mapping API prototypes

function FmxMapFileAccessModeToFlags(Value: TFmxFileAccessMode): UINT;
function FmxMapFlagsToFileAccessMode(Value: UINT): TFmxFileAccessMode;
function FmxMapFileSharingModeToFlags(Value: TFmxFileSharingMode): UINT;
function FmxMapFlagsToFileSharingMode(Value: UINT): TFmxFileSharingMode;
function FmxMapFileOpenOptionsToFlags(Value: TFmxFileOpenOptions): UINT;
function FmxMapFlagsToFileOpenOptions(Value: UINT): TFmxFileOpenOptions;

  // Function prototypes

function FmxOpenFile(FileName: TFileName; FileAccessMode: TFmxFileAccessMode; FileSharingMode: TFmxFileSharingMode; FileOpenOptions: TFmxFileOpenOptions; var phfile: TFmxFile): Boolean;
function FmxCloseFile(var phfile: TFmxFile): Boolean;
function FmxReadFile(hfile: TFmxFile; offs: TFmxFilePosition; var v; cb: TFmxFileSize; var pcbRead: TFmxFileSize): Boolean;
function FmxWriteFile(hfile: TFmxFile; offs: TFmxFilePosition; const v; cb: TFmxFileSize; var pcbWritten: TFmxFileSize): Boolean;

function FmxGetFilePos(hfile: TFmxFile): TFmxFilePosition;
function FmxSetFilePos(hfile: TFmxFile; offs: TFmxFilePosition): TFmxFilePosition;
function FmxGetFileSize(hfile: TFmxFile): TFmxFilePosition;
function FmxSetFileSize(hfile: TFmxFile; cbFile: TFmxFileSize): Boolean;
function FmxLock(hfile: TFmxFile; offset: TFmxFilePosition; cb: TFmxFilePosition): Boolean;
function FmxUnlock(hfile: TFmxFile; offset: TFmxFilePosition; cb: TFmxFilePosition): Boolean;
function FmxDeleteFile(FileName: TFileName): Boolean;

  // FmxEnumFiles() callback function prototype
type
  TFmxEnumFilesCallbackProc = function(FileName: TFileName; cbFile: TFmxFileSize; attr: ULONG; pvCallbackData: Pointer): Boolean;

  // FmaEnumFiles() types
type
  TFmxEnumFilesOption = (fmxEnumFilesFullPath);
  TFmxEnumFilesOptions = set of TFmxEnumFilesOption;

  // Mapping API prototypes

function FmxMapFmxEnumFilesOptionsToFlags(Value: TFmxEnumFilesOptions): ULONG;
function FmxMapFlagsToFmxEnumFilesOptions(Value: ULONG): TFmxEnumFilesOptions;

  // Function prototypes

function FmxEnumFiles(Path: AnsiString; Mask: AnsiString; CallbackProc: TFmxEnumFilesCallbackProc;
  pvCallbackData: Pointer; EnumFilesOptions: TFmxEnumFilesOptions): Boolean;

function FmxFileExists(Path: AnsiString): Boolean;
function FmxDirExists(Path: AnsiString): Boolean;
function FmxMakeDir(Path: AnsiString): Boolean;

/////////////////////////////////////////////////////////////////////////////
// Miscellaneous declarations
/////////////////////////////////////////////////////////////////////////////

  // Types used by FmaGetOSType() and FmaGiveUpTimeSlice()

type
  TFmxOSType = (
    fmxOSTypeDOS,
    fmxOSTypeWindows,
    fmxOSTypeWindowsNT,
    fmxOSTypeDesqView,
    fmxOSTypeOS2DOS,
    fmxOSTypeOS2,
    fmxOSTypeWin32,
    fmxOSTypeMacOS,
    fmxOSTypeUnknown
    );

{
const
  OSTYPE_DOS =             $0001;          // Plain DOS
  OSTYPE_WINDOWS =         $0002;          // Microsoft Windows DosBox
  OSTYPE_WINNT =           $0003;          // Microsoft Windows NT DosBox
  OSTYPE_DESQVIEW =        $0004;          // QuarterDesk DesqView
  OSTYPE_OS2DOS =          $0005;          // IBM OS/2 DosBox
  OSTYPE_OS2 =             $0006;          // Native IBM OS/2
  OSTYPE_W32 =             $0007;          // Native Win32
  OSTYPE_MACOS =           $0008;          // Native Mac OS
  OSTYPE_MASK =            $000F;          // OS type mask
  OSTYPE_PROTMODE =        $8000;          // Protected mode
}

  // Function prototypes

procedure FmxInterlockedIncrement(var plv: LONG);
procedure FmxInterlockedDecrement(var plv: LONG);
function FmxGetLocalDateTime(var pDateTime: TDateTime): Boolean;
function FmxGetLocalTime: ULONG;
procedure FmxSleep(milliseconds: ULONG);
function FmxGetMsgIdStamp: ULONG;
function FmxQueryOSType(var ProtectedMode: Boolean): TFmxOSType;
procedure FmxGiveOutTimeSlice;

implementation

uses
  FmxDef
  {$IFDEF FMA_STATIC_LINKAGE}, FmaObj{$ENDIF};

function FmxMapFileAccessModeToFlags(Value: TFmxFileAccessMode): UINT;
const
  Flags: array[TFmxFileAccessMode] of UINT = (
    FOPENFILE_READONLY,
    FOPENFILE_WRITEONLY,
    FOPENFILE_READWRITE
    );
begin
  Result := Flags[Value];
end;

function FmxMapFlagsToFileAccessMode(Value: UINT): TFmxFileAccessMode;
begin
  if (Value and FOPENFILE_READWRITE) = FOPENFILE_READWRITE then
  begin
    Result := fmxFileReadWrite;
  end
  else
  begin
    if (Value and FOPENFILE_READONLY) = FOPENFILE_READONLY then
    begin
      Result := fmxFileReadOnly;
    end
    else
    begin
      if (Value and FOPENFILE_WRITEONLY) = FOPENFILE_WRITEONLY then
      begin
        Result := fmxFileWriteOnly;
      end
      else
      begin
        raise EConvertError.Create('FmxMapFlagsToFileAccessMode: No file access flags defined.');
      end;
    end;
  end;
end;

function FmxMapFileSharingModeToFlags(Value: TFmxFileSharingMode): UINT;
const
  Flags: array[TFmxFileSharingMode] of UINT = (
    FOPENFILE_DENYNONE,
    FOPENFILE_DENYALL
    );
begin
  Result := Flags[Value];
end;

function FmxMapFlagsToFileSharingMode(Value: UINT): TFmxFileSharingMode;
begin
  if (Value and FOPENFILE_DENYNONE) = FOPENFILE_DENYNONE then
  begin
    Result := fmxFileDenyNone;
  end
  else
  begin
    Result := fmxFileDenyAll;
  end;
end;

function FmxMapFileOpenOptionsToFlags(Value: TFmxFileOpenOptions): UINT;
begin
  Result := 0;
  if fmxFileCreate in Value then
  begin
    Result := Result or FOPENFILE_CREATE;
  end;
  if fmxFileTruncate in Value then
  begin
    Result := Result or FOPENFILE_TRUNCATE;
  end;
end;

function FmxMapFlagsToFileOpenOptions(Value: UINT): TFmxFileOpenOptions;
begin
  Result := [];
  if (Value and FOPENFILE_CREATE) = FOPENFILE_CREATE then
  begin
    Include(Result, fmxFileCreate);
  end;
  if (Value and FOPENFILE_TRUNCATE) = FOPENFILE_TRUNCATE then
  begin
    Include(Result, fmxFileTruncate);
  end;
end;

function FmxOpenFile(FileName: TFileName; FileAccessMode: TFmxFileAccessMode; FileSharingMode: TFmxFileSharingMode; FileOpenOptions: TFmxFileOpenOptions; var phfile: TFmxFile): Boolean;
var
  fOpen: UINT;
begin
  fOpen :=  FmxMapFileAccessModeToFlags(FileAccessMode) or
            FmxMapFileSharingModeToFlags(FileSharingMode) or
            FmxMapFileOpenOptionsToFlags(FileOpenOptions);
  Result := FmaOpenFile(PAnsiChar(FileName), fOpen, phfile);
end;

function FmxCloseFile(var phfile: TFmxFile): Boolean;
begin
  Result := FmaCloseFile(phfile);
end;

function FmxReadFile(hfile: TFmxFile; offs: TFmxFilePosition; var v; cb: TFmxFileSize; var pcbRead: TFmxFileSize): Boolean;
begin
  Result := FmaReadFile(hfile, offs, @v, cb, pcbRead);
end;

function FmxWriteFile(hfile: TFmxFile; offs: TFmxFilePosition; const v; cb: TFmxFileSize; var pcbWritten: TFmxFileSize): Boolean;
begin
  Result := FmaWriteFile(hfile, offs, @v, cb, pcbWritten);
end;

function FmxGetFilePos(hfile: TFmxFile): TFmxFilePosition;
begin
  Result := FmaGetFilePos(hfile);
end;

function FmxSetFilePos(hfile: TFmxFile; offs: TFmxFilePosition): TFmxFilePosition;
begin
  Result := FmaSetFilePos(hfile, offs);
end;

function FmxGetFileSize(hfile: TFmxFile): TFmxFilePosition;
begin
  Result := FmaGetFileSize(hfile);
end;

function FmxSetFileSize(hfile: TFmxFile; cbFile: TFmxFileSize): Boolean;
begin
  Result := FmaSetFileSize(hfile, cbFile);
end;

function FmxLock(hfile: TFmxFile; offset: TFmxFilePosition; cb: TFmxFilePosition): Boolean;
begin
  Result := FmaLock(hfile, offset, cb, True);
end;

function FmxUnlock(hfile: TFmxFile; offset: TFmxFilePosition; cb: TFmxFilePosition): Boolean;
begin
  Result := FmaLock(hfile, offset, cb, False);
end;

function FmxDeleteFile(FileName: TFileName): Boolean;
begin
  Result := FmaDeleteFile(PAnsiChar(FileName));
end;

function FmxMapFmxEnumFilesOptionsToFlags(Value: TFmxEnumFilesOptions): ULONG;
begin
  Result := 0;
  if fmxEnumFilesFullPath in Value then
  begin
    Result := Result or FENUMFILES_FULLPATH;
  end;
end;

function FmxMapFlagsToFmxEnumFilesOptions(Value: ULONG): TFmxEnumFilesOptions;
begin
  Result := [];
  if (Value and FENUMFILES_FULLPATH) = FENUMFILES_FULLPATH then
  begin
    Include(Result, fmxEnumFilesFullPath);
  end;
end;

type
  TEnumFilesCallbackAdaptorSettings = record
    FCallbackProc: TFmxEnumFilesCallbackProc;
    FCallbackData: Pointer;
  end;
  PEnumFilesCallbackAdaptorSettings = ^TEnumFilesCallbackAdaptorSettings;

function EnumFilesCallbackAdaptor(pszFile: PAnsiChar; cbFile: ULONG; attr: ULONG; pvCallbackData: Pointer): BOOL; stdcall;
var
  FileName: TFileName;
begin
  FileName := StrPas(pszFile);
  with PEnumFilesCallbackAdaptorSettings(pvCallbackData)^ do
  begin
    Result := FCallbackProc(FileName, cbFile, attr, FCallbackData);
  end;
end;

function FmxEnumFiles(Path: AnsiString; Mask: AnsiString; CallbackProc: TFmxEnumFilesCallbackProc;
  pvCallbackData: Pointer; EnumFilesOptions: TFmxEnumFilesOptions): Boolean;
var
  CallbackAdaptorSettings: TEnumFilesCallbackAdaptorSettings;
begin
  CallbackAdaptorSettings.FCallbackProc := CallbackProc;
  CallbackAdaptorSettings.FCallbackData := pvCallbackData;
  Result := FmaEnumFiles(
    PAnsiChar(Path), PAnsiChar(Mask),
    @EnumFilesCallbackAdaptor, @CallbackAdaptorSettings,
    FmxMapFmxEnumFilesOptionsToFlags(EnumFilesOptions)
    );
end;

function FmxFileExists(Path: AnsiString): Boolean;
begin
  Result := FmaFileExists(PAnsiChar(Path));
end;

function FmxDirExists(Path: AnsiString): Boolean;
begin
  Result := FmaDirExists(PAnsiChar(Path));
end;

function FmxMakeDir(Path: AnsiString): Boolean;
begin
  Result := FmaMakeDir(PAnsiChar(Path));
end;

procedure FmxInterlockedIncrement(var plv: LONG);
begin
  FmaInterlockedIncrement(plv);
end;

procedure FmxInterlockedDecrement(var plv: LONG);
begin
  FmaInterlockedDecrement(plv);
end;

function FmxGetLocalDateTime(var pDateTime: TDateTime): Boolean;
var
  dttm: FmaDef.DTTM;
begin
  Result := FmaGetLocalDttm(dttm);
  pDateTime := DTTMtoDateTime(dttm);
end;

function FmxGetLocalTime: ULONG;
begin
  Result := FmaGetLocalTime();
end;

procedure FmxSleep(milliseconds: ULONG);
begin
  FmaSleep(milliseconds);
end;

function FmxGetMsgIdStamp: ULONG;
begin
  Result := FmaGetMsgIdStamp();
end;

function FmxQueryOSType(var ProtectedMode: Boolean): TFmxOSType;
const
  OSTypes: array[OSTYPE_DOS..OSTYPE_MACOS] of TFmxOSType = (
    fmxOSTypeDOS,
    fmxOSTypeWindows,
    fmxOSTypeWindowsNT,
    fmxOSTypeDesqView,
    fmxOSTypeOS2DOS,
    fmxOSTypeOS2,
    fmxOSTypeWin32,
    fmxOSTypeMacOS
    );
var
  FmaOSType: UINT;
begin
  FmaOSType := FmaQueryOSType();
  ProtectedMode := ((FmaOSType and OSTYPE_PROTMODE) = OSTYPE_PROTMODE);
  FmaOSType := FmaOSType and OSTYPE_MASK;
  if (FmaOSType >= Low(OSTypes)) and (FmaOSType <= High(OSTypes)) then
  begin
    Result := OSTypes[FmaOSType];
  end
  else
  begin
    Result := fmxOSTypeUnknown;
  end;
end;

procedure FmxGiveOutTimeSlice;
begin
  FmaGiveOutTimeSlice();
end;

end.
