unit FmxUti;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: FmaUti Utilities that references Delphi functionality
//
// FmxUti.Pas
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 17/Feb/2000
// Updated: 17/Feb/2000
//
// History: 17/Feb/2000 akzhan  - Initially created for Delphi convertion.
//
/////////////////////////////////////////////////////////////////////////////

interface

uses
  Windows, FmaDef, FmaUti;

/////////////////////////////////////////////////////////////////////////////
// String manipulation declarations
/////////////////////////////////////////////////////////////////////////////

  // Function prototypes

function FmxGetHash(s: AnsiString): ULONG;

/////////////////////////////////////////////////////////////////////////////
// Dynamic string management declarations
/////////////////////////////////////////////////////////////////////////////

  // Function prototypes

procedure InitPDSZ(var pdsz: PDSZ);
procedure DestroyPDSZ(var pdsz: PDSZ);

function StrToPDSZ(s: AnsiString): PDSZ;
function PDSZToStr(var pdsz: PDSZ): AnsiString;
function PDSZToStrLeave(pdsz: PDSZ): AnsiString;

procedure AddStrToPDSZ(pdsz: PDSZ; s: AnsiString);

/////////////////////////////////////////////////////////////////////////////
// Network address routines declarations
/////////////////////////////////////////////////////////////////////////////

  // Function prototypes

function FmxScanNetAddr(var pnetAddr: NETADDR; s: AnsiString): Integer;
function FmxFormatNetAddr(const pnetAddr: NETADDR; Domain: AnsiString): AnsiString;

implementation

uses
  SysUtils;

function FmxGetHash(s: AnsiString): ULONG;
begin
  Result := FmaGetHash(PAnsiChar(s));
end;

procedure InitPDSZ(var pdsz: PDSZ);
begin
  New(pdsz);
  pdsz^ := DSZ_INIT;
end;

procedure DestroyPDSZ(var pdsz: PDSZ);
begin
  if Assigned(pdsz) then
  begin
    FmaDelDsz(pdsz);
    Dispose(pdsz);
    pdsz := nil;
  end;
end;

function StrToPDSZ(s: AnsiString): PDSZ;
begin
  InitPDSZ(Result);
  if s <> '' then FmaSetDsz(Result, PAnsiChar(s), Length(s));
end;

function PDSZToStr(var pdsz: PDSZ): AnsiString;
begin
  if Assigned(pdsz) then
  begin
    if pdsz^.ich > 0 then
    begin
      Result := pdsz^.psz;
    end
    else
    begin
      Result := '';
    end;
    DestroyPDSZ(pdsz);
  end;
end;

function PDSZToStrLeave(pdsz: PDSZ): AnsiString;
begin
  if Assigned(pdsz) and (pdsz^.ich > 0) then
  begin
    Result := pdsz^.psz;
  end
  else
  begin
    Result := '';
  end;
end;

procedure AddStrToPDSZ(pdsz: PDSZ; s: AnsiString);
begin
  FmaAddDsz(pdsz, PAnsiChar(s), Length(s));
end;

function FmxScanNetAddr(var pnetAddr: NETADDR; s: AnsiString): Integer;
var
  head, tail: PAnsiChar;
begin
  Result := 0;
  if s <> '' then
  begin
    head := PAnsiChar(s);
    tail := FmaScanNetAddr(pnetAddr, head);
    if Assigned(tail) then Result := (tail - head) + 1;
  end;
end;

function FmxFormatNetAddr(const pnetAddr: NETADDR; Domain: AnsiString): AnsiString;
// 65535:6553565535/65535.65535@domain = 29 + length(domain)
begin
  SetLength(Result, 29 + Length(Domain));
  FmaFormatNetAddr(PAnsiChar(Result), Length(Result), pnetAddr, PAnsiChar(Domain));
  Result := Trim(Result);
end;

end.
