unit JamApi;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: JAM api declarations module
//
// JamApi.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 27/Aug/1999
// Updated: 18/Feb/2000
//
// History: 07/Sep/1998 kvitekp - Initially created for MBS project.
//          18/Feb/2000 akzhan  - Convertion complete.
//
/////////////////////////////////////////////////////////////////////////////
//
// JAM(mbp) - Copyright 1993 Joaquim Homrighausen,
//            Andrew Milner, Mats Birch, Mats Wallin.
//            ALL RIGHTS RESERVED.
//
// Note: These declarations were derived from original
//       specifications published by JAM message base
//       authors in JAM(mbp) API documentation.
//
/////////////////////////////////////////////////////////////////////////////
//
// NOTE: ALL STRUCTURES IN THIS MODULE ASSUME SINGLE BYTE PACKING !!!
//
/////////////////////////////////////////////////////////////////////////////
{$I FmxDef.inc}
interface

uses
  Windows, FmaDef, FmaApi;

  // JAM base file extensions

const
  JAM_JHR_FILEEXT = '.JHR'; // Header file extension
  JAM_JDT_FILEEXT = '.JDT'; // Text file extension
  JAM_JDX_FILEEXT = '.JDX'; // Index file extension
  JAM_JLR_FILEEXT = '.JLR'; // Lastread file extension

  // JAM structures magic number and revision level

const
  JAM_MAGIC = $004D414A; // Equals to string <J><A><M><nul>
  JAM_REVLEVEL = 1;          // Current JAM revision level

  // JAM base header structure at the beginning of <area>.jhr

type
  _JBH = packed record (* jbh *)
    magic: ULONG;              // Magic number JAM_MAGIC
    time: ULONG;               // Creation time
    cmod: ULONG;               // Modification counter
    cmsgActive: ULONG;         // Number of active (not deleted) msgs
    crcp: ULONG;               // CRC-32 of password to access
    msgnBase: TMsgNumber;           // Lowest message number in index file
    rsvd: packed array[0..999] of Byte;         // --- reserved
  end;
  JBH = _JBH;
  PJBH = ^JBH;

  // JAM message header structure in <area>.jhr

type
  _JMH = packed record (* jmh *)
    magic: ULONG;              // Magic number (JAM_MAGIC)
    revLevel: USHORT;           // Current revision level (JAM_REVLEVEL)
    rsvd: USHORT;               // --- reserved
    cbjsf: ULONG;              // Length of subfields
    cTimesRead: ULONG;         // Number of times message was read
    crcMsgId: ULONG;           // CRC-32 of MSGID line
    crcReply: ULONG;           // CRC-32 of REPLY line
    msgnReplyTo: TMsgNumber;        // This msg is a reply to
    msgnReply1st: TMsgNumber;       // First reply to this msg
    msgnReplyNext: TMsgNumber;      // Next msg in reply chain
    timeWritten: ULONG;        // When msg was written
    timeReceived: ULONG;       // When msg was received/read
    timeProcessed: ULONG;      // When msg was processed by packer
    msgn: TMsgNumber;               // Message number (1-based)
    attr: ULONG;               // Msg attributes (JMA_)
    attr2: ULONG;              // --- reserved attributes
    offText: ULONG;            // Offset of text in <area>.JDT
    cchText: ULONG;            // Length of message text in <area>.JDT
    crcp: ULONG;               // CRC-32 of password to access msg
    cost: ULONG;               // Cost of message
  end;
  JMH = _JMH;
  PJMH = ^JMH;

  // JAM message attributes in jmh.attr

const
  JMA_LOCAL       = $00000001;     // Msg created locally
  JMA_INTRANSIT   = $00000002;     // Msg is in-transit
  JMA_PRIVATE     = $00000004;     // Private message
  JMA_READ        = $00000008;     // Read by addressee
  JMA_SENT        = $00000010;     // Sent to remote
  JMA_KILLSENT    = $00000020;     // Kill when sent
  JMA_ARCHIVESENT = $00000040;     // Archive when sent
  JMA_HOLD        = $00000080;     // Hold for pick-up
  JMA_CRASH       = $00000100;     // Crash
  JMA_IMMEDIATE   = $00000200;     // Send Msg now, ignore restrictions
  JMA_DIRECT      = $00000400;     // Send directly to destination
  JMA_GATE        = $00000800;     // Send via gateway
  JMA_FILEREQUEST = $00001000;     // File request
  JMA_FILEATTACH  = $00002000;     // File(s) attached to Msg
  JMA_TRUNCFILE   = $00004000;     // Truncate file(s) when sent
  JMA_KILLFILE    = $00008000;     // Delete file(s) when sent
  JMA_RECEIPTREQ  = $00010000;     // Return receipt requested
  JMA_CONFIRMREQ  = $00020000;     // Confirmation receipt requested
  JMA_ORPHAN      = $00040000;     // Unknown destination
  JMA_ENCRYPT     = $00080000;     // Msg text is encrypted
  JMA_COMPRESS    = $00100000;     // Msg text is compressed
  JMA_ESCAPED     = $00200000;     // Msg text is seven bit ASCII
  JMA_FPU         = $00400000;     // Force pickup
  JMA_TYPELOCAL   = $00800000;     // Msg is for local use only (not for export)
  JMA_TYPEECHO    = $01000000;     // Msg is for conference distribution
  JMA_TYPENET     = $02000000;     // Msg is direct network mail
  JMA_0x04000000  = $04000000;     // -- reserved
  JMA_0x08000000  = $08000000;     // -- reserved
  JMA_0x10000000  = $10000000;     // -- reserved
  JMA_NODISP      = $20000000;     // Msg may not be displayed to user
  JMA_LOCKED      = $40000000;     // Msg is locked, no editing possible
  JMA_DELETED     = $80000000;     // Msg is deleted

  // JAM message subfield structure

type
  _JSF = packed record (* jsf *)
    id: USHORT;                    // Subfield ID, 0-65535
    id2: USHORT;                   // Reserved for future use
    cb: ULONG;                     // Subfield data length
  // ab: array[0..cb - 1] of Byte; // Subfield data bytes
  end;
  JSF = _JSF;
  PJSF = ^JSF;

function PJSFDATA(pjsf: PJSF): PByte; // Poiner to subfield data

  // JAM message subfield types

const
  JSF_OADDRESS   = 0;       // Originating address          100 bytes max
  JSF_DADDRESS   = 1;       // Destination address          100 bytes max
  JSF_SENDERNAME = 2;       // Author's name                100 bytes max
  JSF_RECVRNAME  = 3;       // Recipient's name             100 bytes max
  JSF_MSGID      = 4;       // Message identification data  100 bytes max
  JSF_REPLYID    = 5;       // Message reply data           100 bytes max
  JSF_SUBJECT    = 6;       // Message subject              100 bytes max
  JSF_PID        = 7;       // FTN PID kludge line           40 bytes max
  JSF_TRACE      = 8;       // FTN ^aVia information
  JSF_ENCLFILE   = 9;       // File attachment
  JSF_ENCLFWALIAS= 10;      // File attachment asciiz alias
  JSF_ENCLFREQ   = 11;      // File request
  JSF_ENCLFILEWC = 12;      // One or more file attachment (wild cards)
  JSF_ENCLINDFILE= 13;      // File attachment reference file
  JSF_EMBINDAT   = 1000;    // Reserved for future use
  JSF_FTSKLUDGE  = 2000;    // FTS-compliant "kludge" line
  JSF_SEENBY2D   = 2001;    // 2D (net/node) SEEN-BY information
  JSF_PATH2D     = 2002;    // 2D (net/node) PATH information
  JSF_FLAGS      = 2003;    // FTN FLAGS kludge information
  JSF_TZUTCINFO  = 2004;    // Time zone information
  JSF_UNKNOWN    = $ffff;   // Unknown subfield type

  // JAM index file record structure

type
  _JDX = packed record (* jdx *)
    crc: ULONG;                // CRC-32 of destination username
    lmh: ULONG;                // Offset of header in .JHR file
  end;
  JDX = _JDX;
  PJDX = ^JDX;

  // JAM index empty entry initialization

const
  JDX_INIT_EMPTY: JDX = ( crc: ULONG(-1); lmh: ULONG(-1) );

  // JAM lastread file record structure

type
  _JLR = packed record (* jlr *)
    crc: ULONG;                // CRC-32 of user name (lowercase)
    uid: ULONG;                // Unique UserID
    msgnLast: TMsgNumber;      // Last read message number
    msgnHigh: TMsgNumber;      // Highest read message number
  end;
  JLR = _JLR;
  PJLR = ^JLR;

  // JAM message base locking of <area>.JHR

const
  JBH_LOCK_OFFS  = 0;       // Locking region offset
  JBH_LOCK_LENG  = 1;       // Locking region length
  JBH_LOCK_RETRY = 5;       // Max number of locking attempts

/////////////////////////////////////////////////////////////////////////////
// JAM api declarations
/////////////////////////////////////////////////////////////////////////////

  // Special values used by JamReadMsg()

const
  JRM_GETJSFCOPY =  0;          // *pccbjsf : Allocate a copy of subfields info
  JRM_GETJSFPTR  = ULONG(-1);   // *pccbjsf : Return a readonly pointer to subfields info
  JRM_GETALLTEXT = ULONG(-1);   // *pcchText: read entire message text

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function JamGetApi: PAPI; stdcall;

function JamOpenArea(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle; stdcall;

function JamCloseArea(harea: TAreaHandle): BOOL; stdcall;
function JamLockArea(harea: TAreaHandle; bLock: BOOL): BOOL; stdcall;

function JamValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL; stdcall;
function JamCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL; stdcall;
function JamEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL; stdcall;

function JamDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function JamSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function JamGetCurMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function JamGetFirstMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function JamGetHighMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function JamGetMsgCount(harea: TAreaHandle): LONG; stdcall;

function JamMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG; stdcall;
function JamUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber; stdcall;

function JamOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle; stdcall;
function JamCloseMsg(hmsg: TMsgHandle): BOOL; stdcall;

function JamLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL; stdcall;
function JamSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL; stdcall;

function JamReadMsg(hmsg: TMsgHandle; pjmh: PJMH; var ppjsf: PJSF; pcbjsf: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL; stdcall;

function JamWriteMsg(hmsg: TMsgHandle; pjmh: PJMH; pjsf: PJSF; cbjsf: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL; stdcall;

function JamGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL; stdcall;
function JamGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL; stdcall;

function JamGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL; stdcall;
function JamSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PAnsiChar): BOOL; stdcall;
{$ENDIF}

implementation

function PJSFDATA(pjsf: PJSF): PByte; // Poiner to subfield data
begin
// # define PJSFDATA(pjsf) ((PBYTE)(pjsf) + sizeof(JSF))
  Result := PByte(PAnsiChar(pjsf) + sizeof(JSF));
end;

{$IFNDEF FMA_STATIC_LINKAGE}
function JamGetApi: PAPI;
  external FmaDll;

function JamOpenArea(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle;
  external FmaDll;

function JamCloseArea(harea: TAreaHandle): BOOL;
  external FmaDll;
function JamLockArea(harea: TAreaHandle; bLock: BOOL): BOOL;
  external FmaDll;

function JamValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL;
  external FmaDll;
function JamCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL;
  external FmaDll;
function JamEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL;
  external FmaDll;

function JamDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL;
  external FmaDll;
function JamSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL;
  external FmaDll;
function JamGetCurMsg(harea: TAreaHandle): TMsgNumber;
  external FmaDll;
function JamGetFirstMsg(harea: TAreaHandle): TMsgNumber;
  external FmaDll;
function JamGetHighMsg(harea: TAreaHandle): TMsgNumber;
  external FmaDll;
function JamGetMsgCount(harea: TAreaHandle): LONG;
  external FmaDll;

function JamMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG;
  external FmaDll;
function JamUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber;
  external FmaDll;

function JamOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle;
  external FmaDll;
function JamCloseMsg(hmsg: TMsgHandle): BOOL;
  external FmaDll;

function JamLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL;
  external FmaDll;
function JamSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL;
  external FmaDll;

function JamReadMsg(hmsg: TMsgHandle; pjmh: PJMH; var ppjsf: PJSF; pcbjsf: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL;
  external FmaDll;

function JamWriteMsg(hmsg: TMsgHandle; pjmh: PJMH; pjsf: PJSF; cbjsf: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL;
  external FmaDll;

function JamGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL;
  external FmaDll;
function JamGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL;
  external FmaDll;

function JamGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL;
  external FmaDll;
function JamSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PAnsiChar): BOOL;
  external FmaDll;
{$ENDIF}
end.
