unit PktApi;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Pkt api declarations module
//
// PktApi.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 08/Feb/2000
// Updated: 14/Oct/2002
//
// History: 08/Feb/2000 kvitekp - Initially created.
//          18/Feb/2000 akzhan  - Convertion complete.
//          14/Oct/2002 Plyako A- Some bugs of convertion fixed.
//
/////////////////////////////////////////////////////////////////////////////
//
// Note: These declaratons were derived from original specifications
//       published in FTS-0001.16, FSC-0039.004 and FSC-0048.002 and
//       some common real life practices.
//
/////////////////////////////////////////////////////////////////////////////
//
// NOTE: ALL STRUCTURES IN THIS MODULE ASSUME SINGLE BYTE PACKING !!!
//
/////////////////////////////////////////////////////////////////////////////
{$I FmxDef.inc}
interface
uses
  Windows, FmaDef, FmaApi, FtsApi;

  // Packet file extensions

const
  PKT_PKT_FILEEXT = '.PKT'; // Packet file extension

  // Packet header structure (FSC-0048.002)

type
  _PBH = packed record (* pbh *)
    origNode: USHORT;           // Origination node number
    destNode: USHORT;           // Destination node number
    year: USHORT;               // Year of packet creation (e.g. 1999)
    month: USHORT;              // Month of packet creation (0-11 for Jan-Dec)
    day: USHORT;                // Day of packet creation (1-31)
    hour: USHORT;               // Hour of packet creation (0-23)
    minute: USHORT;             // Minute of packet creation (0-59)
    second: USHORT;             // Second of packet creation (0-59)
    baud: USHORT;               // Max baud rate of orig and dest
    _type: USHORT;              // Packet type (PKT_TYPE2)
    origNet: USHORT;            // Origination net number
    destNet: USHORT;            // Destination net number
    prodCodeLow: BYTE;          // Product code (low)
    prodRevMajor: BYTE;         // Product revision (major)
    achPassword: packed array[0..7] of AnsiChar; // Password (null padded)
    origZone0: USHORT;          // Origination zone number (ZMail,QMail)
    destZone0: USHORT;          // Destination zone number (ZMail,QMail)
    auxNet: USHORT;             // Auxiliary net number
    capWordCopy: USHORT;        // Byte swapped copy of capability word
    prodCodeHigh: BYTE;         // Product code (high)
    prodRevMinor: BYTE;         // Product revision (minor)
    capWord: USHORT;            // Capability word
    origZone: USHORT;           // Origination zone number (FrontDoor)
    destZone: USHORT;           // Destination zone number (FrontDoor)
    origPoint: USHORT;          // Origination point number
    destPoint: USHORT;          // Destination point number
    achData: packed array[0..3] of AnsiChar; // Product specific data
  end;
  PBH = _PBH;
  PPBH = ^PBH;

  // Packet type

const
  PKT_TYPE2     =  $0002;  // Always version 2

  // Packet capability word

const
  PCAP_TYPE2PLUS = $0001; // Type-2+ packet
  PCAP_RFC822    = $8000; // Allows RFC-822

  // FMA product code... to be ascquired from FSC

const
  FMA_PRODCODE = $1234;    // None for now

  // Packed message header structure mostly used

type
  _PMH = packed record (* pmh *)
    _type: USHORT;              // Packed message type  (PMH_TYPE2)
    origNode: USHORT;           // Origination node number
    destNode: USHORT;           // Destination node number
    origNet: USHORT;            // Origination net number
    destNet: USHORT;            // Destination net number
    attr: USHORT;               // Message attributes (FMA_)
    cost: USHORT;               // Unit cost charged to send the message
    achDate: packed array[0..19] of AnsiChar; // ASCII date information
  //  CHAR    achTo[max 36];      // To name (asciiz)
  //  CHAR    achBy[max 36];      // From name (asciiz)
  //  CHAR    achRe[max 72];      // Message subject (asciiz)
  end;
  PMH = _PMH;
  PPMH = ^PMH;

  // Packed message stamp

const
  PMH_TYPE2  = $0002;      // Always version 2

  // Packet file locking

const
  PBH_LOCK_OFFS  = 0;       // Locking region offset
  PBH_LOCK_LENG  = 1;       // Locking region length
  PBH_LOCK_RETRY = 5;       // Max number of locking attempts

/////////////////////////////////////////////////////////////////////////////
// Pkt api declarations
/////////////////////////////////////////////////////////////////////////////

  // Special values used by PktReadMsg()

const
  PRM_GETCTRLCOPY =         0;      // *pcchCtrl: Allocate a copy of msg control info
  PRM_GETCTRLPTR  = ULONG(-1);     // *pcchCtrl: Return a readonly pointer to msg control info
  PRM_GETALLTEXT  = ULONG(-1);     // *pcchText: read entire message text

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function PktGetApi: PAPI; stdcall;

function PktOpenArea(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle; stdcall;

function PktCloseArea(harea: TAreaHandle): BOOL; stdcall;
function PktLockArea(harea: TAreaHandle; bLock: BOOL): BOOL; stdcall;

function PktValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL; stdcall;
function PktCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL; stdcall;
function PktEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL; stdcall;

function PktDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function PktSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function PktGetCurMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function PktGetFirstMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function PktGetHighMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function PktGetMsgCount(harea: TAreaHandle): LONG; stdcall;

function PktMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG; stdcall;
function PktUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber; stdcall;

function PktOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle; stdcall;
function PktCloseMsg(hmsg: TMsgHandle): BOOL; stdcall;

function PktLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL; stdcall;
function PktSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL; stdcall;

function PktReadMsg(hmsg: TMsgHandle; pfmh: PFMH; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL; stdcall;

function PktWriteMsg(hmsg: TMsgHandle; pfmh: PFMH; pchCtrl: PAnsiChar; cchCtrl: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL; stdcall;

function PktGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL; stdcall;
function PktGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL; stdcall;

function PktGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL; stdcall;
function PktSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PAnsiChar): BOOL; stdcall;
{$ENDIF}
implementation

{$IFNDEF FMA_STATIC_LINKAGE}
function PktGetApi: PAPI; 
  external FmaDll name 'PktGetApi';

function PktOpenArea(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle; 
  external FmaDll name 'PktOpenArea';

function PktCloseArea(harea: TAreaHandle): BOOL;
  external FmaDll name 'PktCloseArea';
function PktLockArea(harea: TAreaHandle; bLock: BOOL): BOOL; 
  external FmaDll name 'PktLockArea';

function PktValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL; 
  external FmaDll name 'PktValidateMsg';
function PktCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL; 
  external FmaDll name 'PktCreateMsgMap';
function PktEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL; 
  external FmaDll name 'PktEnumMsgs';

function PktDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; 
  external FmaDll name 'PktDeleteMsg';
function PktSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; 
  external FmaDll name 'PktSetCurMsg';
function PktGetCurMsg(harea: TAreaHandle): TMsgNumber; 
  external FmaDll name 'PktGetCurMsg';

function PktGetFirstMsg(harea: TAreaHandle): TMsgNumber;
  external FmaDll name 'PktGetFirstMsg';

function PktGetHighMsg(harea: TAreaHandle): TMsgNumber; 
  external FmaDll name 'PktGetHighMsg';
function PktGetMsgCount(harea: TAreaHandle): LONG; 
  external FmaDll name 'PktGetMsgCount';

function PktMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG; 
  external FmaDll name 'PktMsgnToUid';
function PktUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber; 
  external FmaDll name 'PktUidToMsgn';

function PktOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle; 
  external FmaDll name 'PktOpenMsg';
function PktCloseMsg(hmsg: TMsgHandle): BOOL; 
  external FmaDll name 'PktCloseMsg';

function PktLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL; 
  external FmaDll name 'PktLoadMsg';

function PktSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL;
  external FmaDll name 'PktSaveMsg';

function PktReadMsg(hmsg: TMsgHandle; pfmh: PFMH; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL; 
  external FmaDll name 'PktReadMsg';

function PktWriteMsg(hmsg: TMsgHandle; pfmh: PFMH; pchCtrl: PAnsiChar; cchCtrl: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL; 
  external FmaDll name 'PktWriteMsg';

function PktGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL; 
  external FmaDll name 'PktGetMsgCtrlLen';
function PktGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL; 
  external FmaDll name 'PktGetMsgTextLen';

function PktGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL; 
  external FmaDll name 'PktGetLRInfo';
function PktSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PAnsiChar): BOOL; 
  external FmaDll name 'PktSetLRInfo';

{$ENDIF}

end.

