unit SquApi;
/////////////////////////////////////////////////////////////////////////////
//
// Fidonet Mail Access: Squish api declarations module
//
// SquApi.h
//
// Copyright (c) 1999-2000 by Pete Kvitek
// Convertion to Delphi's Object Pascal by Akzhan Abdulin
//                                         http://www.akzhan.midi.ru/fma/
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version. See GNU_GPL.TXT for details.
//
// Created: 27/Aug/1999
// Updated: 18/Feb/2000
//
// History: 07/Sep/1998 kvitekp - Initially created for MBS project.
//          18/Feb/2000 akzhan  - Convertion complete.
//
/////////////////////////////////////////////////////////////////////////////
//
// Squish base - Copyright 1989, 1995 by Lanius Corporation.
//               All rights reserved.
//
// Note: These declarations were derived from original
//       specifications published by Squish message base
//       author Scott Dudley in Squish Developers Kit v2.00
//
/////////////////////////////////////////////////////////////////////////////
//
// NOTE: ALL STRUCTURES IN THIS MODULE ASSUME SINGLE BYTE PACKING !!!
//
/////////////////////////////////////////////////////////////////////////////
{$I FmxDef.inc}
interface

uses
  Windows, FmaDef, FmaApi;

  // Squish base file extensions

const
  SQU_SQD_FILEEXT = '.SQD'; // Data file extension
  SQU_SQI_FILEEXT = '.SQI'; // Index file extension
  SQU_SQL_FILEEXT = '.SQL'; // Lastread file extension

  // Squish base header structure at the beginning of <area>.sqd

type
  _SBH = packed record (* sbh *)
    cb: USHORT;                        // Size of the message base header
    rsvd1: USHORT;                     // --- reserved
    cmsg: ULONG;                       // Number of messages in area
    msgnHigh: ULONG;                   // Highest msg in area. Same as cmsg.
    cmsgSkip: ULONG;                   // Number of first messages to keep
    umsgHwtr: UMSG;                    // Hight water mark message unique id
    umsgNext: UMSG;                    // Next unique message id to use
    achPath: packed array[0..79] of AnsiChar; // Message base path
    lfmFirst: LONG;                    // First frame offset
    lfmLast: LONG;                     // Last frame offset
    lfmFree: LONG;                     // First free frame offset
    lfmLastFree: LONG;                 // Last free frame offset
    lfmFrameEnd: LONG;                 // Frame end offset (eof)
    cmsgMax: ULONG;                    // Max number of messages to keep
    keepDays: USHORT;                  // Number of days to keep message
    cbSfh: USHORT;                     // Size of the squish frame header
    rsvd2: packed array[0..123] of AnsiChar;  // --- reserved for future use
  end;
  SBH = _SBH;
  PSBH = ^SBH;

  // Squish frame header which goes before each and every message

type
  _SFH = packed record (* sfh *)
    magic: ULONG;             // Magic number SFH_MAGIC
    lfmNext: LONG;            // Next frame offset
    lfmPrev: LONG;            // Previous frame offset
    cbFrame: ULONG;           // Frame size not including this header
    cbMsg: ULONG;             // Msg size (msg header, ctrl info and text)
    cbCtrl: ULONG;            // Message control info size
    _type: USHORT;            // Frame type (FTYPE_)
    rsvd: USHORT;             // --- reserved for future use
  end;
  SFH = _SFH;
  PSFH = ^SFH;

  // Squish frame header magic number in sfh.magic

const
  SFH_MAGIC     =  $AFAE4453;

  // Squish frame types in sqfh.type

const
  FRAME_USED      = $0000;  // Used frame
  FRAME_FREE      = $0001;  // Free frame
  FRAME_LZSS      = $0002;  // N/A
  FRAME_UPDATE    = $0003;  // Frame is being updated

  // Squish message header structure

type
  _SMH = packed record (* smh *)
    attr: ULONG;                       // Message attributes (SMA_)
    achBy: packed array[0..35] of AnsiChar;   // From name string
    achTo: packed array[0..35] of AnsiChar;   // To name string
    achRe: packed array[0..71] of AnsiChar;   // Subject string
    netOrig: TFmaNetAddr;              // Origination address
    netDest: TFmaNetAddr;              // Destination address
    dttmWritten: DTTM;                 // Written date/time (UTC)
    dttmArrived: DTTM;                 // Arrived date/time (UTC)
    utcOffset: SHORT;                  // Author's UTC offset in minutes
    umsgReplyTo: UMSG;                 // Original message uid
    aumsgReplies: array[0..8] of UMSG; // Array  of the reply msgs
    umsg: UMSG;                        // Msg uid if (attr & SMA_UMSG)
    achDate: packed array[0..19] of AnsiChar; // Obsolete date information
  end;
  SMH = _SMH;
  PSMH = ^SMH;

  // Squish message attributes in smh.attr

const
  SMA_PRVT     = $0001;   // Private                 +
  SMA_CRSH     = $0002;   // Crash                   | These
  SMA_READ     = $0004;   // Read                    | flags are
  SMA_SENT     = $0008;   // Sent                    | standard for
  SMA_FILE     = $0010;   // File attach             | FidoNet(tm)
  SMA_TRAN     = $0020;   // Transit                 | message attr
  SMA_ORPH     = $0040;   // Orphan                  |
  SMA_KILL     = $0080;   // Kill/sent               |
  SMA_LOCL     = $0100;   // Local                   |
  SMA_HOLD     = $0200;   // Hold                    |
  SMA_RSV2     = $0400;   // Reserved                |
  SMA_FREQ     = $0800;   // File request            |
  SMA_RREQ     = $1000;   // Return receipt request  |
  SMA_RRCP     = $2000;   // Confirmation receipt msg|
  SMA_AREQ     = $4000;   // Audit request           |
  SMA_UREQ     = $8000;   // File update request     +
  SMA_SCAN = $00010000;  // Squish scanned          - *.SQ? only
  SMA_UMSG = $00020000;  // Valid xmsg.umsg         - *.SQ? only

  SMA_XSQU = $0003ffff;  // Known squish message attributes

  // Squish message index structure

type
  _SQI = packed record (* sqi *)
    lfm: LONG;                 // Frame offset in <area>.sqd
    umsg: UMSG;                // Message unique id
    hash: ULONG;               // To name hash value
  end;
  SQI = _SQI;
  PSQI = ^SQI;

  // Squish index read message hash flag

const
  SQI_HASH_READ =  $80000000;

  // Squish index empty entry initialization

const
  SQI_INIT_EMPTY: SQI = (lfm: 0; umsg: ULONG(-1); hash: ULONG(-1));

  // Squish lastread file record structure

type
  _SLR = packed record (* slr *)
  	umsg: UMSG;               // Last read message uid
  end;
  SLR = _SLR;
  PSLR = ^SLR;

  // Squish message base locking of <area>.sqd

const
  SBH_LOCK_OFFS  = 0;       // Locking region offset
  SBH_LOCK_LENG  = 1;       // Locking region length
  SBH_LOCK_RETRY = 5;       // Max number of locking attempts

/////////////////////////////////////////////////////////////////////////////
// Squish api declarations
/////////////////////////////////////////////////////////////////////////////

  // Special values used by SquReadMsg()

const
  SRM_GETCTRLCOPY = 0;         // *pcchCtrl: Allocate a copy of msg control info
  SRM_GETCTRLPTR  = ULONG(-1); // *pcchCtrl: Return a readonly pointer to msg control info
  SRM_GETALLTEXT  = ULONG(-1); // *pcchText: read entire message text

  // Function prototypes
{$IFNDEF FMA_STATIC_LINKAGE}
function SquGetApi: PAPI; stdcall;

function SquOpenArea(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle; stdcall;

function SquCloseArea(harea: TAreaHandle): BOOL; stdcall;
function SquLockArea(harea: TAreaHandle; bLock: BOOL): BOOL; stdcall;

function SquValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL; stdcall;
function SquCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL; stdcall;
function SquEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL; stdcall;

function SquDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function SquSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL; stdcall;
function SquGetCurMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function SquGetFirstMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function SquGetHighMsg(harea: TAreaHandle): TMsgNumber; stdcall;
function SquGetMsgCount(harea: TAreaHandle): LONG; stdcall;

function SquMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG; stdcall;
function SquUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber; stdcall;

function SquOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle; stdcall;
function SquCloseMsg(hmsg: TMsgHandle): BOOL; stdcall;

function SquLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL; stdcall;
function SquSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL; stdcall;

function SquReadMsg(hmsg: TMsgHandle; psmh: PSMH; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL; stdcall;

function SquWriteMsg(hmsg: TMsgHandle; psmh: PSMH; pchCtrl: PAnsiChar; cchCtrl: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL; stdcall;

function SquGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL; stdcall;
function SquGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL; stdcall;

function SquGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL; stdcall;
function SquSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PChar): BOOL; stdcall;
{$ENDIF}
implementation
{$IFNDEF FMA_STATIC_LINKAGE}
function SquGetApi: PAPI;
  external FmaDll;

function SquOpenArea(pszPath: PAnsiChar; fOpenMode: UINT; fl: ULONG; var pnetAddr: NETADDR): TAreaHandle;
  external FmaDll;

function SquCloseArea(harea: TAreaHandle): BOOL;
  external FmaDll;
function SquLockArea(harea: TAreaHandle; bLock: BOOL): BOOL;
  external FmaDll;

function SquValidateMsg(harea: TAreaHandle; msgn: TMsgNumber; pimsg: PULONG): BOOL;
  external FmaDll;
function SquCreateMsgMap(harea: TAreaHandle; var paumsg: PUMSG; var pcumsg: ULONG): BOOL;
  external FmaDll;
function SquEnumMsgs(harea: TAreaHandle; pfnCallback: PFNENUMMSGS; pvCallbackData: Pointer; fl: ULONG): BOOL;
  external FmaDll;

function SquDeleteMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL;
  external FmaDll;
function SquSetCurMsg(harea: TAreaHandle; msgn: TMsgNumber): BOOL;
  external FmaDll;
function SquGetCurMsg(harea: TAreaHandle): TMsgNumber;
  external FmaDll;
function SquGetFirstMsg(harea: TAreaHandle): TMsgNumber;
  external FmaDll;
function SquGetHighMsg(harea: TAreaHandle): TMsgNumber;
  external FmaDll;
function SquGetMsgCount(harea: TAreaHandle): LONG;
  external FmaDll;

function SquMsgnToUid(harea: TAreaHandle; msgn: TMsgNumber): UMSG;
  external FmaDll;
function SquUidToMsgn(harea: TAreaHandle; umsg: UMSG; fMode: UINT): TMsgNumber;
  external FmaDll;

function SquOpenMsg(harea: TAreaHandle; mode: UINT; msgn: TMsgNumber; fl: ULONG): TMsgHandle;
  external FmaDll;
function SquCloseMsg(hmsg: TMsgHandle): BOOL;
  external FmaDll;

function SquLoadMsg(hmsg: TMsgHandle; var ppfmm: PFMM; fl: ULONG): BOOL;
  external FmaDll;
function SquSaveMsg(hmsg: TMsgHandle; pfmm: PFMM; fl: ULONG; cchTextTotal: ULONG): BOOL;
  external FmaDll;

function SquReadMsg(hmsg: TMsgHandle; psmh: PSMH; var ppchCtrl: PAnsiChar; pcchCtrl: PULONG;
  var ppchText: PAnsiChar; pcchText: PULONG; poffText: PULONG): BOOL;
  external FmaDll;

function SquWriteMsg(hmsg: TMsgHandle; psmh: PSMH; pchCtrl: PAnsiChar; cchCtrl: ULONG;
  pchText: PAnsiChar; cchText: ULONG; poffText: PULONG; cchTextTotal: ULONG): BOOL;
  external FmaDll;

function SquGetMsgCtrlLen(hmsg: TMsgHandle; var pcchCtrl: ULONG): BOOL;
  external FmaDll;
function SquGetMsgTextLen(hmsg: TMsgHandle; var pcchText: ULONG): BOOL;
  external FmaDll;

function SquGetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO): BOOL;
  external FmaDll;
function SquSetLRInfo(harea: TAreaHandle; uid: ULONG; plri: PLRINFO; pszUserName: PChar): BOOL;
  external FmaDll;
{$ENDIF}
end.
